package com.baidu.cloud.videoplayer.demo;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Toast;

import com.baidu.cloud.media.player.feature.FeatureException;
import com.baidu.cloud.media.player.feature.FeatureManager;
import com.baidu.cloud.videoplayer.demo.feed.FeedActivity;

public class EntryActivity extends Activity implements View.OnClickListener {
    private static final String TAG = EntryActivity.class.getSimpleName();
    private static final String RTC_LOGIN_ACTIVITY_CLASS_NAME
            = "com.baidu.cloud.videoplayer.demo.rtc.RTCPlayerLoginActivity";
    private static final String VIRTUAL_LIVE_ACTIVITY_CLASS_NAME
            = "com.baidu.cloud.videoplayer.demo.virtuallive.VirtualLiveActivity";

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_entry);

        findViewById(R.id.view_play_common).setOnClickListener(this);
        findViewById(R.id.view_play_rtc).setOnClickListener(this);
        findViewById(R.id.view_play_feed).setOnClickListener(this);
        findViewById(R.id.view_virtual_live).setOnClickListener(this);
    }


    @Override
    public void onClick(View v) {
        int id = v.getId();
        if (id == R.id.view_play_rtc) {
            try {
                boolean enable =  FeatureManager.getInstance()
                        .checkRtcPlayerFeatureValid(this, BaseApp.APP_ID);
                if (enable) {
                    jumpToRtcPlayActivity();
                }
            } catch (FeatureException e) {
                Log.e(TAG, e.getFeatureErrMsg());
                if (e.getFeatureErrCode() == FeatureException.FEATURE_ERR_LICENSE_INVALID) {
                    Toast.makeText(this, "当前License不支持超低延时直播", Toast.LENGTH_SHORT).show();
                } else if (e.getFeatureErrCode() == FeatureException.FEATURE_ERR_MODULE_INVALID) {
                    Toast.makeText(this, "当前SDK不包含超低延时直播功能", Toast.LENGTH_SHORT).show();
                }
            }
        } else if (id == R.id.view_play_common) {
            jumpToCommonPlayActivity();
        } else if (id == R.id.view_play_feed) {
            jumpToFeedPlayActivity();
        } else if (id == R.id.view_virtual_live) {
            try {
                boolean enable =  FeatureManager.getInstance()
                        .checkVirtualLiveFeatureValid(this, BaseApp.APP_ID);
                if (enable) {
                    jumpToVirtualLiveActivity();
                }
            } catch (FeatureException e) {
                Log.e(TAG, e.getFeatureErrMsg());
                if (e.getFeatureErrCode() == FeatureException.FEATURE_ERR_LICENSE_INVALID) {
                    Toast.makeText(this, "当前License不支持绿幕抠像", Toast.LENGTH_SHORT).show();
                } else if (e.getFeatureErrCode() == FeatureException.FEATURE_ERR_MODULE_INVALID) {
                    Toast.makeText(this, "当前SDK不包含绿幕抠像功能", Toast.LENGTH_SHORT).show();
                }
            }
        }
    }

    private void jumpToCommonPlayActivity() {
        Intent liveIntent = new Intent(this, SampleListActivity.class);
        startActivity(liveIntent);
    }

    private void jumpToFeedPlayActivity() {
        FeedActivity.start(this, 0);
    }

    private void jumpToRtcPlayActivity() {
        try {
            Class clazzRtcLoginActivity = Class.forName(RTC_LOGIN_ACTIVITY_CLASS_NAME);
            Intent liveIntent = new Intent(this, clazzRtcLoginActivity);
            startActivity(liveIntent);
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }
    }

    private void jumpToVirtualLiveActivity() {
        try {
            Class clazzVirtualLiveActivity = Class.forName(VIRTUAL_LIVE_ACTIVITY_CLASS_NAME);
            Intent liveIntent = new Intent(this, clazzVirtualLiveActivity);
            startActivity(liveIntent);
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }
    }
}
