package com.baidu.cloud.videoplayer.demo.bar;

import android.app.Activity;
import android.content.Context;
import android.graphics.Bitmap;
import android.os.Environment;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.text.TextUtils;
import android.util.AttributeSet;
import android.util.Log;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.SeekBar;
import android.widget.SeekBar.OnSeekBarChangeListener;
import android.widget.TextView;
import android.widget.Toast;

import com.baidu.cloud.media.player.BDCloudMediaPlayer;
import com.baidu.cloud.media.player.feature.FeatureException;
import com.baidu.cloud.media.player.render.record.IRecordListener;
import com.baidu.cloud.videoplayer.demo.BaseApp;
import com.baidu.cloud.videoplayer.demo.C;
import com.baidu.cloud.videoplayer.demo.R;
import com.baidu.cloud.videoplayer.demo.helper.AutoScrollTextView;
import com.baidu.cloud.videoplayer.demo.helper.CommonUtils;
import com.baidu.cloud.videoplayer.demo.info.SharedPrefsStore;
import com.baidu.cloud.videoplayer.demo.popview.DefaultPopWindow;
import com.baidu.cloud.videoplayer.widget.BDCloudVideoView;
import com.baidu.cloud.videoplayer.widget.BDCloudVideoView.PlayerState;
import com.baidu.cloud.videoplayer.demo.danmu.DefaultDanmuView;
import com.baidu.cloud.projectionengine.DeviceUpnp;
import com.baidu.cloud.projectionengine.ProjectionEngine;
import com.baidu.cloud.projectionengine.javadlna.ProjectionRender;
import com.bumptech.glide.Glide;
import com.bumptech.glide.load.engine.DiskCacheStrategy;
import com.bumptech.glide.request.animation.GlideAnimation;
import com.bumptech.glide.request.target.SimpleTarget;

import java.io.File;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.Timer;
import java.util.TimerTask;

/**
 * 自定义播放控制类
 *
 * @author baidu
 */
public class AdvancedMediaController extends RelativeLayout implements OnClickListener,
        Handler.Callback {

    private static final String TAG = "AdvancedMediaController";

    private ImageButton playButton;
    private ImageButton snapshotButton;
    private Button fitButton;
    private ImageButton nextButton;
    private ImageButton previousButton;
    private Button resolutionButton;
    private AutoScrollTextView audioEffectButton;
    private TextView speedButton;
    private ImageButton downloadButton;
    private Button smartVisionButton;
    private TextView positionView;
    private SeekBar seekBar;
    private TextView durationView;
    private TextView infoResolutionView;
    // private TextView infoBitrateView;
    private TextView infoNetworkView;
    private EditText danmuEditText;
    private TextView sendDanmuButton;
    private TextView switchDanmuButton;
    private TextView castButton;
    private DefaultPopWindow castPopWindow;

    private ArrayList<DeviceUpnp> mCastReceivers = new ArrayList<>();
    private boolean bFindingCastReceiver = false;
    private boolean bCasting = false;
    private int mCastReceiverIdx = -1;
    private int mCastButtonWidth;
    private OnCastStateListener mOnCastStateListener;
    private int mCastState = OnCastStateListener.CAST_END;
    private static final int MSG_CANCEL_CAST_SEARCH = 900;

    private String[] availableResolution = null;

    private int mSmartVisionType;
    private static final String SV_HDR = "HDR";
    private static final String[] smartVision = {
            SV_HDR,
    };

    private static final String AF_CLOSE = "关闭音效";
    private static final String AF_SPEAKER_ORIGINAL = "扬声器原声模式";
    private static final String AF_SPEAKER_MUSIC = "扬声器音乐模式";
    private static final String AF_SPEAKER_MOVIE = "扬声器电影模式";
    private static final String AF_EARPHONE_ORIGINAL = "耳机原声模式";
    private static final String AF_EARPHONE_MUSIC = "耳机音乐模式";
    private static final String AF_EARPHONE_MOVIE = "耳机电影模式";
    private static final String AF_EARPHONE_SURROUND = "耳机全景环绕模式";
    private static final String AF_SA_ORIGINAL_PHONE = "空间音频原声模式"; // 手机端
    private static final String AF_SA_MUSIC_PHONE = "空间音频音乐模式"; // 手机端
    private static final String AF_SA_MOVIE_PHONE = "空间音频电影模式"; // 手机端
    private static final String[] demoAudioEffect = {
            AF_CLOSE,
            AF_SPEAKER_ORIGINAL,
            AF_SPEAKER_MUSIC,
            AF_SPEAKER_MOVIE,
            AF_EARPHONE_ORIGINAL,
            AF_EARPHONE_MUSIC,
            AF_EARPHONE_MOVIE,
            AF_EARPHONE_SURROUND,
            AF_SA_ORIGINAL_PHONE,
            AF_SA_MOVIE_PHONE,
            AF_SA_MUSIC_PHONE
    };

    private static final String SP_X1 = "x1";
    private static final String SP_X1_25 = "x1.25";
    private static final String SP_X1_5 = "x1.5";
    private static final String SP_X2 = "x2";
    private static final String SP_X0_75 = "x0.75";
    private static final String SP_X0_5 = "x0.5";
    private static final String[] demoSpeed = {
            SP_X1,
            SP_X1_25,
            SP_X1_5,
            SP_X2,
            SP_X0_75,
            SP_X0_5
    };

    private Handler mainThreadHandler = new Handler(Looper.getMainLooper(), this);

    private BDCloudVideoView mVideoView = null;
    private DefaultDanmuView mDanmuView = null;

    private ImageView mPreImageView = null;
    private TextView mTimeView = null;

    private String[] mPreImages = null;

    boolean bIsDragging = false;

    private int lastIndex = -1;
    private Bitmap lastBitmap = null;

    private View.OnClickListener preListener;
    private View.OnClickListener nextListener;
    private View.OnClickListener downloadListener;

    private Timer positionTimer;
    private static final int POSITION_REFRESH_TIME = 500;

    private boolean bLocked;
    private boolean bMaxSetted = false;
    private long currentPositionInMilliSeconds = 0L;

    private SimpleTarget target = new SimpleTarget<Bitmap>() {
        @Override
        public void onResourceReady(Bitmap bitmap, GlideAnimation glideAnimation) {
            lastBitmap = bitmap;
        }
    };

    public AdvancedMediaController(Context context, AttributeSet attrs) {
        super(context, attrs);
        initUI();
    }

    public AdvancedMediaController(Context context) {
        super(context);
        initUI();
    }

    private void initUI() {
        bLocked = false;
        // inflate controller bar
        LayoutInflater inflater = (LayoutInflater) this.getContext().getSystemService(Activity.LAYOUT_INFLATER_SERVICE);
        final View layout = inflater.inflate(R.layout.bar_advanced_media_controller, this);

        playButton = layout.findViewById(R.id.ibtn_play);
        playButton.setOnClickListener(this);

        snapshotButton = layout.findViewById(R.id.ibtn_snapshot);
        snapshotButton.setOnClickListener(this);

        fitButton = layout.findViewById(R.id.btn_fitmode);
        if (SharedPrefsStore.isPlayerFitModeCrapping(this.getContext())) {
            fitButton.setText("裁剪");
        } else {
            fitButton.setText("填充");
        }
        fitButton.setOnClickListener(this);

        previousButton = layout.findViewById(R.id.ibtn_previous);
        previousButton.setOnClickListener(this);

        nextButton = layout.findViewById(R.id.ibtn_next);
        nextButton.setOnClickListener(this);

        resolutionButton = layout.findViewById(R.id.btn_resolution);
        resolutionButton.setOnClickListener(this);

        audioEffectButton = layout.findViewById(R.id.btn_audioeffect);
        audioEffectButton.setOnClickListener(this);
        audioEffectButton.setText(demoAudioEffect[0]);

        speedButton = layout.findViewById(R.id.btn_speed);
        speedButton.setOnClickListener(this);
        speedButton.setText(demoSpeed[0]);

        downloadButton = layout.findViewById(R.id.ibtn_download);
        downloadButton.setOnClickListener(this);
        
        smartVisionButton = (Button) layout.findViewById(R.id.btn_smart_vision);
        smartVisionButton.setOnClickListener(this);
        mSmartVisionType = CommonUtils.getSmartVisionType(getContext());
        switch (mSmartVisionType) {
            case C.SMARTVISION_HDR:
                smartVisionButton.setVisibility(VISIBLE);
                smartVisionButton.setText(SV_HDR);
                break;
            case C.SMARTVISION_DISABLE:
            default:
                smartVisionButton.setVisibility(GONE);
                break;
        }

        positionView = layout.findViewById(R.id.tv_position);
        seekBar = layout.findViewById(R.id.seekbar);
        seekBar.setMax(0);

        durationView = layout.findViewById(R.id.tv_duration);

        infoResolutionView = layout.findViewById(R.id.tv_resolution);
        // infoBitrateView = layout.findViewById(R.id.tv_bitrate);
        infoNetworkView = layout.findViewById(R.id.tv_netspeed);

        layout.findViewById(R.id.tv_start_record).setOnClickListener(this);
        layout.findViewById(R.id.tv_stop_record).setOnClickListener(this);

        danmuEditText = layout.findViewById(R.id.et_danmu);
        sendDanmuButton = layout.findViewById(R.id.btn_send_danmu);
        sendDanmuButton.setOnClickListener(this);
        switchDanmuButton = layout.findViewById(R.id.btn_danmu_switch);
        switchDanmuButton.setOnClickListener(this);
        castButton = layout.findViewById(R.id.tv_cast);
        castButton.setOnClickListener(this);

        seekBar.setOnSeekBarChangeListener(new OnSeekBarChangeListener() {
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
                if (bIsDragging == true && mPreImages[0] != null) {
                    if (lastIndex != spriteImageIndex(progress)) {
                        lastIndex = spriteImageIndex(progress);
                        if (lastIndex < mPreImages.length) {
                            Glide.with(getContext())
                                    .load(mPreImages[lastIndex])
                                    .asBitmap()
                                    .diskCacheStrategy(DiskCacheStrategy.SOURCE)
                                    .into(target);
                        }
                    }
                    mTimeView.setText(CommonUtils.stringForTime(progress));
                    if (lastBitmap != null) {
                        int position = thumbnailPosition(progress);
                        int row = position / 3;
                        int coln = position % 3;
                        Bitmap bitmap = Bitmap.createBitmap(lastBitmap, coln * 231, row * 130, 231, 130);
                        mPreImageView.setImageBitmap(bitmap);
                    }
                }
                updatePostion(progress);
            }

            public void onStartTrackingTouch(SeekBar seekBar) {
                bIsDragging = true;
                if (mPreImages[0] != null) {
                    mPreImageView.setVisibility(VISIBLE);
                    mTimeView.setVisibility(VISIBLE);
                }
            }

            public void onStopTrackingTouch(SeekBar seekBar) {
                if (mPreImages[0] != null) {
                    mPreImageView.setVisibility(INVISIBLE);
                    mTimeView.setVisibility(INVISIBLE);
                }
                if (bCasting && mCastReceiverIdx >= 0) {
                    ProjectionEngine.getInstance().seek(mCastReceiverIdx,
                            seekBar.getProgress() / 1000);
                } else {
                    if (mVideoView.getDuration() > 0) {
                        // 仅非直播的视频支持拖动
                        currentPositionInMilliSeconds = seekBar.getProgress();
                        if (mVideoView != null) {
                            mVideoView.seekTo(seekBar.getProgress());
                        }
                    }
                    if (mDanmuView != null) {
                        mDanmuView.seekTo((long)seekBar.getProgress());
                    }
                }

                bIsDragging = false;
            }
        });
        enableControllerBar(false);
    }

    public void seek(int progress) {
        seekBar.setProgress(progress);
        if (mVideoView.getDuration() > 0) {
            // 仅非直播的视频支持拖动
            currentPositionInMilliSeconds = seekBar.getProgress();
            if (mVideoView != null) {
                mVideoView.seekTo(seekBar.getProgress());
            }
        }
        if (mDanmuView != null) {
            mDanmuView.seekTo((long)seekBar.getProgress());
        }
    }

    public int getDuration() {
        return mVideoView.getDuration();
    }

    public int getCurrentPosition() {
        return mVideoView.getCurrentPosition();
    }

    public void setSpeed(float speed) {
        if (mVideoView != null) {
            mVideoView.setSpeed(speed);
        }
        if (mDanmuView != null) {
            mDanmuView.setSpeed(speed);
        }
    }

    public float getSpeed() {
        if (mVideoView != null) {
            return mVideoView.getSpeed();
        }
        return 1.0f;
    }

    public void togglePlayPause() {
        if (mVideoView != null) {
            if (mVideoView.isPlaying()) {
                mVideoView.pause();
            } else  {
                mVideoView.start();
            }
        }
    }

    public View getVideoView() {
        return mVideoView;
    }

    public void toggleLock() {
        seekBar.setEnabled(bLocked);
        bLocked = !bLocked;
    }

    public boolean isLocked() {
        return bLocked;
    }

    public boolean isPlaying() {
        if (mVideoView != null) {
            return mVideoView.isPlaying();
        }
        return false;
    }

    public void changeState() {
        final PlayerState state = mVideoView.getCurrentPlayerState();
        Log.d(TAG, "mediaController: changeStatus=" + state.name());
        bMaxSetted = false;
        mainThreadHandler.post(new Runnable() {

            @Override
            public void run() {
                if (state == PlayerState.STATE_IDLE || state == PlayerState.STATE_ERROR) {
                    stopPositionTimer();
                    if (!bCasting) {
                        playButton.setEnabled(true);
                        playButton.setBackgroundResource(R.drawable.toggle_btn_play);
                        seekBar.setEnabled(false);
                    }
                    resolutionButton.setEnabled(false);
                    audioEffectButton.setEnabled(false);
                    speedButton.setEnabled(false);
                    updatePostion(mVideoView == null ? 0 : mVideoView.getCurrentPosition());
                    updateDuration(mVideoView == null ? 0 : mVideoView.getDuration());
                } else if (state == PlayerState.STATE_PREPARING) {
                    playButton.setEnabled(false);
//                    playButton.setBackgroundResource(R.drawable.toggle_btn_play);
                    seekBar.setEnabled(false);
                    resolutionButton.setEnabled(false);
                    speedButton.setEnabled(false);
                } else if (state == PlayerState.STATE_PREPARED) {

                    playButton.setEnabled(true);
                    playButton.setBackgroundResource(R.drawable.toggle_btn_play);
                    seekBar.setEnabled(true);

                    // set width
                    String resolutionStr = mVideoView.getVideoWidth() + "x" + mVideoView.getVideoHeight();
                    if (mVideoView.getVideoWidth() > 0) { // 可能为音频文件，width=-1
                        infoResolutionView.setText(resolutionStr);
                    }

                    String[] variantInfo = mVideoView.getVariantInfo();
                    if (mVideoView.getMediaSwitchType() == BDCloudMediaPlayer.SOURCE_SWITCH_SMOOTH_MODE) {
                        variantInfo = mVideoView.getMutilMediasDsc();
                    }

                    setAvailableResolution(variantInfo);
                    updateDuration(mVideoView == null ? 0 : mVideoView.getDuration());
                    seekBar.setMax(mVideoView.getDuration());

                    resolutionButton.setEnabled(true);
                    resolutionButton.setText(getDescriptionOfResolution(resolutionStr));
                    audioEffectButton.setEnabled(true);
                    speedButton.setEnabled(true);

                    mCastButtonWidth = castButton.getWidth();
                } else if (state == PlayerState.STATE_PLAYBACK_COMPLETED) {
                    stopPositionTimer();
                    seekBar.setProgress(seekBar.getMax());
                    seekBar.setEnabled(false);
                    playButton.setEnabled(true);
                    playButton.setBackgroundResource(R.drawable.toggle_btn_play);
//                    resolutionButton.setEnabled(false);
                } else if (state == PlayerState.STATE_PLAYING) {
                    startPositionTimer();
                    seekBar.setEnabled(true);
                    playButton.setEnabled(true);
                    playButton.setBackgroundResource(R.drawable.toggle_btn_pause);
                } else if (state == PlayerState.STATE_PAUSED) {
                    stopPositionTimer();
                    playButton.setEnabled(true);
                    playButton.setBackgroundResource(R.drawable.toggle_btn_play);
                } else if (state == PlayerState.STATE_VIDEOSIZE_CHANGED) {
                    String resolutionStr = mVideoView.getVideoWidth() + "x" + mVideoView.getVideoHeight();
                    if (mVideoView.getVideoWidth() > 0) {
                        infoResolutionView.setText(resolutionStr);
                    }
                }
            }

        });

    }

    private void startPositionTimer() {
        if (positionTimer != null) {
            positionTimer.cancel();
            positionTimer = null;
        }
        positionTimer = new Timer();
        positionTimer.schedule(new TimerTask() {

            @Override
            public void run() {
                mainThreadHandler.post(new Runnable() {
                    @Override
                    public void run() {
                        onPositionUpdate();

                        // update speed
                        infoNetworkView.setText((mVideoView.getDownloadSpeed() >> 10) + "KB/s");
                    }
                });

            }

        }, 0, POSITION_REFRESH_TIME);
    }

    private void stopPositionTimer() {
        if (positionTimer != null) {
            positionTimer.cancel();
            positionTimer = null;
        }
    }

    /**
     * if you don't want to use Controller when still playing, invoke release to stop timer!
     */
    public void release() {
        stopPositionTimer();
    }

    public void setMediaPlayerControl(BDCloudVideoView player) {
        mVideoView = player;
    }

    public void setDanmuControl(DefaultDanmuView danmuControl) {
        mDanmuView = danmuControl;
    }

    public void setViewImage(ImageView imageView, String[] preImages) {
        mPreImageView = imageView;
        mPreImages = preImages;
    }

    public void setTextView(TextView textView) {
        mTimeView = textView;
    }

    /**
     * Show the controller on screen. It will go away automatically after 3
     * seconds of inactivity.
     */
    public void show() {
        if (mVideoView == null) {
            return;
        }
        setProgress((int) currentPositionInMilliSeconds);

        this.setVisibility(View.VISIBLE);
    }

    /**
     * Remove the controller from the screen.
     */
    public void hide() {
        this.setVisibility(View.GONE);
    }

    private void updateDuration(int milliSecond) {
        if (durationView != null) {
            durationView.setText(CommonUtils.stringForTime(milliSecond));
        }
    }

    private void updatePostion(int milliSecond) {
        if (positionView != null) {
            positionView.setText(CommonUtils.stringForTime(milliSecond));
        }
    }

    private int spriteImageIndex(int milliSecond) {
        return ((milliSecond / 1000) / (1 * 6));
    }

    private int thumbnailPosition(int milliSecond) {
        return (milliSecond / 1000) % (1 * 6);
    }

    /**
     * Set the max process for the seekBar, usually the lasting milliseconds
     */
    public void setMax(int maxProgress) {
        if (bMaxSetted) {
            return;
        }
        if (seekBar != null) {
            seekBar.setMax(maxProgress);
        }
        updateDuration(maxProgress);
        if (maxProgress > 0) {
            bMaxSetted = true;
        }
    }

    /**
     * in milliSeconds
     */
    public void setProgress(int progress) {
        if (seekBar != null) {
            seekBar.setProgress(progress);
        }
    }

    /**
     * in milliseconds
     */
    public void setCache(int cache) {
        if (seekBar != null && cache != seekBar.getSecondaryProgress()) {
            seekBar.setSecondaryProgress(cache);
        }
    }

    public void enableControllerBar(boolean isEnable) {
        seekBar.setEnabled(isEnable);
        playButton.setEnabled(isEnable);
    }

    /**
     * onPositionUpdate is invoked per 500ms
     */
    public boolean onPositionUpdate() {
        if (mVideoView == null) {
            return false;
        }
        long newPositionInMilliSeconds = mVideoView.getCurrentPosition();
//        Log.d(TAG, "onPositionUpdate getCurrentPosition=" + newPositionInMilliSeconds);
        long previousPosition = currentPositionInMilliSeconds;
        if (newPositionInMilliSeconds > 0 && !bIsDragging) {
            currentPositionInMilliSeconds = newPositionInMilliSeconds;
        }
        if (getVisibility() != View.VISIBLE) {
            // 如果控制条不可见，则不设置进度
            return false;
        }
        if (!bIsDragging) {
            int durationInMilliSeconds = mVideoView.getDuration();
            if (durationInMilliSeconds > 0) {
                this.setMax(durationInMilliSeconds);
                // 直播视频的duration为0，此时不设置进度
                if (previousPosition != newPositionInMilliSeconds) {
                    this.setProgress((int) newPositionInMilliSeconds);
                }
            }
        }
        return false;
    }

    public void onTotalCacheUpdate(final long milliSeconds) {
        mainThreadHandler.post(new Runnable() {

            @Override
            public void run() {
                setCache((int) milliSeconds);
            }

        });
    }

    @Override
    public void onClick(View v) {
        if (bLocked) {
            return;
        }
        switch (v.getId()) {
            case R.id.ibtn_play:
                if (bCasting && mCastReceiverIdx >= 0) {
                    if (mCastState == OnCastStateListener.CAST_START) {
                        playButton.setBackgroundResource(R.drawable.toggle_btn_play);
                        ProjectionEngine.getInstance().pause(mCastReceiverIdx);
                    } else {
                        playButton.setBackgroundResource(R.drawable.toggle_btn_pause);
                        ProjectionEngine.getInstance().play(mCastReceiverIdx, 1);
                    }
                } else {
                    if (mVideoView == null) {
                        Log.d(TAG, "playButton checkstatus changed, but bindView=null");
                    } else {
                        if (mVideoView.isPlaying()) {
                            Log.d(TAG, "playButton: Will invoke pause()");
                            playButton.setBackgroundResource(R.drawable.toggle_btn_play);
                            mVideoView.pause();
                        } else {
                            Log.d(TAG, "playButton: Will invoke resume()");
                            playButton.setBackgroundResource(R.drawable.toggle_btn_pause);
                            mVideoView.start();
                        }
                    }
                }
                break;
            case R.id.ibtn_snapshot:
                // take snapshot
                File sdDir = null;
                String strpath = null;
                Bitmap bitmap = null;
                Date date = new Date();
                SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd-HH-mm-ss-SSSS");
                String time = formatter.format(date);

                // TODO 您可以指定存储路径，以下逻辑是保存在sdcard根目录下
                sdDir = Environment.getExternalStorageDirectory();
                strpath = sdDir.toString() + "/" + time + ".jpg";
                Log.d(TAG, "snapshot save path=" + strpath);

                bitmap = mVideoView.getBitmap();
                if (bitmap != null) {
                    FileOutputStream os = null;
                    try {
                        os = new FileOutputStream(strpath, false);
                        bitmap.compress(Bitmap.CompressFormat.JPEG, 100, os);
                        os.flush();
                        os.close();
                    } catch (Throwable t) {
                        t.printStackTrace();
                    } finally {
                        if (os != null) {
                            try {
                                os.close();
                            } catch (Throwable t) {
                                Log.e(TAG, "Error occurred while saving snapshot!");
                            }
                        }
                    }
                    os = null;
                    Toast.makeText(this.getContext(), "截图保存在sd卡根目录下, 文件名为" + strpath, Toast.LENGTH_SHORT).show();
                } else {
                    Toast.makeText(this.getContext(), "抱歉，当前无法截图", Toast.LENGTH_SHORT).show();
                }


                break;
            case R.id.btn_fitmode:
                if (fitButton.getText().equals("填充")) {
                    // 转为 裁剪模式：视频保持比例缩放，不留黑边，填满显示区域的两边
                    mVideoView.setVideoScalingMode(BDCloudVideoView.VIDEO_SCALING_MODE_SCALE_TO_FIT_WITH_CROPPING);
                    fitButton.setText("裁剪");
                    SharedPrefsStore.setPlayerFitMode(getContext(), true);
                } else {
                    // 转为 填充模式：视频保持比例缩放，至少一边与显示区域相同，另一边有黑边
                    mVideoView.setVideoScalingMode(BDCloudVideoView.VIDEO_SCALING_MODE_SCALE_TO_FIT);
                    fitButton.setText("填充");
                    SharedPrefsStore.setPlayerFitMode(getContext(), false);
                }
                break;
            case R.id.ibtn_previous:
                if (preListener != null) {
                    preListener.onClick(v);
                }
                break;
            case R.id.ibtn_next:
                if (nextListener != null) {
                    nextListener.onClick(v);
                }
                break;
            case R.id.btn_resolution:
                // select a resolution
                Log.v(TAG, "Show resolution clicked");
                if (this.availableResolution != null && this.availableResolution.length > 0) {

                    OnItemClickListener listener = new OnItemClickListener() {

                        @Override
                        public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                            if (mVideoView != null) {
//                            mVideoView.selectResolutionType(position);
                                mVideoView.selectResolutionByIndex(position);
                                resolutionButton.setText(availableResolution[position]);
                                // infoBitrateView.setText(BVideoView.getSupportedBitrateKb()[position]
                                // / 1000 + "Kbps");
                            }
                        }

                    };
                    // popup window
                    DefaultPopWindow popWindow =
                            DefaultPopWindow.createOptionListPopWindow((Activity) getContext(),
                                    availableResolution, listener);
                    popWindow.showPopupWindow(((Activity) getContext()).findViewById(R.id.root));
                } else {
                    Toast.makeText(this.getContext(), "该视频不是多码率视频(m3u8 master url)", Toast.LENGTH_SHORT).show();
                }
                break;
            case R.id.ibtn_download:

                if (downloadListener != null) {
                    downloadListener.onClick(v);
                }
                break;
            case R.id.tv_start_record:
                if (CommonUtils.getSmartVisionType(getContext()) == C.SMARTVISION_HDR) {
                    Toast.makeText(getContext(), "暂不支持HDR视频的录制", Toast.LENGTH_SHORT).show();
                    break;
                }
                final String filePath = Environment.getExternalStorageDirectory().getPath()
                        + File.separator + new SimpleDateFormat("yyyy-MM-dd-HH-mm-ss").format(new Date()) + ".mp4";
                mVideoView.startRecord(filePath, new IRecordListener() {
                    @Override
                    public void onStartSuccess() {
                        Toast.makeText(getContext(), "录制开始", Toast.LENGTH_SHORT).show();
                    }

                    @Override
                    public void onStopSuccess() {
                        Toast.makeText(getContext(), "录制结束，文件位置：" + filePath, Toast.LENGTH_SHORT).show();
                    }

                    @Override
                    public void onError(int code, String msg) {

                    }

                    @Override
                    public void onProgress(long duration) {
                    
                    }
                });
                break;
            case R.id.tv_stop_record:
                if (CommonUtils.getSmartVisionType(getContext()) == C.SMARTVISION_HDR) {
                    Toast.makeText(getContext(), "暂不支持HDR视频的录制", Toast.LENGTH_SHORT).show();
                    break;
                }
                mVideoView.stopRecord();
                break;
            case R.id.btn_speed:
                if (mVideoView.getDuration() <= 0) {
                    return;
                }
                OnItemClickListener speedListener = new OnItemClickListener() {

                    @Override
                    public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                        if (mVideoView != null) {
                            float speed = 1.0f;
                            switch (demoSpeed[position]) {
                                case SP_X1_5:
                                    speed = 1.5f;
                                    break;
                                case SP_X1_25:
                                    speed = 1.25f;
                                    break;
                                case SP_X2:
                                    speed = 2.0f;
                                    break;
                                case SP_X0_5:
                                    speed = 0.5f;
                                    break;
                                case SP_X0_75:
                                    speed = 0.75f;
                                    break;
                                case SP_X1:
                                default:
                                    speed = 1.0f;
                                    break;
                            }
                            setSpeed(speed);
                            speedButton.setText(demoSpeed[position]);
                        }
                    }

                };
                // popup window
                DefaultPopWindow speedPopWindow =
                        DefaultPopWindow.createOptionListPopWindow((Activity) getContext(),
                                demoSpeed, speedListener);
                speedPopWindow.showPopupWindow(((Activity) getContext()).findViewById(R.id.root));
                break;
            case R.id.btn_send_danmu:
                if (!TextUtils.isEmpty(danmuEditText.getText().toString()) && mDanmuView != null) {
                    mDanmuView.addDanmuItemWithDrawable(danmuEditText.getText().toString(),
                            getResources().getDrawable(R.mipmap.ic_launcher), true);
                    danmuEditText.setText("");
                }
                break;
            case R.id.btn_danmu_switch:
                if (mDanmuView != null) {
                    if (mDanmuView.isVisible()) {
                        mDanmuView.hide();
                        switchDanmuButton.setText("弹幕关");
                    } else {
                        mDanmuView.show();
                        switchDanmuButton.setText("弹幕开");
                    }
                }
                break;
            case R.id.btn_audioeffect:
                    OnItemClickListener afListener = new OnItemClickListener() {

                        @Override
                        public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                            if (mVideoView != null) {
                                try {
                                    switch (demoAudioEffect[position]) {
                                        case AF_SPEAKER_ORIGINAL:
                                            mVideoView.setAudioEffect(BDCloudMediaPlayer.AUDIO_EFFECT.SPEAKER_ORIGINAL);
                                            break;
                                        case AF_SPEAKER_MUSIC:
                                            mVideoView.setAudioEffect(BDCloudMediaPlayer.AUDIO_EFFECT.SPEAKER_MUSIC);
                                            break;
                                        case AF_SPEAKER_MOVIE:
                                            mVideoView.setAudioEffect(BDCloudMediaPlayer.AUDIO_EFFECT.SPEAKER_MOVIE);
                                            break;
                                        case AF_EARPHONE_ORIGINAL:
                                            mVideoView.setAudioEffect(BDCloudMediaPlayer.AUDIO_EFFECT.EARPHONE_ORIGINAL);
                                            break;
                                        case AF_EARPHONE_MUSIC:
                                            mVideoView.setAudioEffect(BDCloudMediaPlayer.AUDIO_EFFECT.EARPHONE_MUSIC);
                                            break;
                                        case AF_EARPHONE_MOVIE:
                                            mVideoView.setAudioEffect(BDCloudMediaPlayer.AUDIO_EFFECT.EARPHONE_MOVIE);
                                            break;
                                        case AF_EARPHONE_SURROUND:
                                            mVideoView.setAudioEffect(BDCloudMediaPlayer.AUDIO_EFFECT.EARPHONE_SURROUND);
                                            break;
                                        case AF_SA_ORIGINAL_PHONE:
                                            mVideoView.setAudioEffect(BDCloudMediaPlayer.AUDIO_EFFECT.SPATIAL_AUDIO_ORIGINAL_PHONE);
                                            if (mVideoView.getBTEarphoneType() != BDCloudMediaPlayer.BT_EARPHONE_TYPE.WANOS_EARPHONE) {
                                                Toast.makeText(getContext(), "需使用空间音频耳机", Toast.LENGTH_SHORT).show();
                                            }
                                            break;
                                        case AF_SA_MUSIC_PHONE:
                                            mVideoView.setAudioEffect(BDCloudMediaPlayer.AUDIO_EFFECT.SPATIAL_AUDIO_MUSIC_PHONE);
                                            if (mVideoView.getBTEarphoneType() != BDCloudMediaPlayer.BT_EARPHONE_TYPE.WANOS_EARPHONE) {
                                                Toast.makeText(getContext(), "需使用空间音频耳机", Toast.LENGTH_SHORT).show();
                                            }
                                            break;
                                        case AF_SA_MOVIE_PHONE:
                                            mVideoView.setAudioEffect(BDCloudMediaPlayer.AUDIO_EFFECT.SPATIAL_AUDIO_MOVIE_PHONE);
                                            if (mVideoView.getBTEarphoneType() != BDCloudMediaPlayer.BT_EARPHONE_TYPE.WANOS_EARPHONE) {
                                                Toast.makeText(getContext(), "需使用空间音频耳机", Toast.LENGTH_SHORT).show();
                                            }
                                            break;
                                        default:
                                            mVideoView.setAudioEffect(BDCloudMediaPlayer.AUDIO_EFFECT.CLOSE);
                                            break;
                                    }
                                } catch (FeatureException e) {
                                    Log.e(TAG, e.getFeatureErrMsg());
                                    if (e.getFeatureErrCode() == FeatureException.FEATURE_ERR_LICENSE_INVALID) {
                                        Toast.makeText(getContext(), "当前License不支持全景声功能", Toast.LENGTH_SHORT).show();
                                    } else if (e.getFeatureErrCode() == FeatureException.FEATURE_ERR_MODULE_INVALID) {
                                        Toast.makeText(getContext(), "当前SDK不包含全景声功能", Toast.LENGTH_SHORT).show();
                                    }
                                }
                                audioEffectButton.setText(demoAudioEffect[position]);
                            }
                        }

                    };
                    // popup window
                    DefaultPopWindow afPopWindow =
                            DefaultPopWindow.createOptionListPopWindow((Activity) getContext(),
                                    demoAudioEffect, 120, afListener);
                    afPopWindow.showPopupWindow(((Activity) getContext()).findViewById(R.id.root));
                break;
            case R.id.btn_smart_vision:
                if (mVideoView != null) {
                    switch (mSmartVisionType) {
                        case C.SMARTVISION_HDR:
                            smartVisionButton.setText(SV_HDR);
                            break;
                    }
                }
                break;
            case R.id.tv_cast:
                if (this.mCastReceivers != null && this.mCastReceivers.size() > 0) {
                    if (bCasting && mCastReceiverIdx >= 0) {
                        ProjectionEngine.getInstance().searchDeviceEnd();
                        ProjectionEngine.getInstance().stop(mCastReceiverIdx);
                        ProjectionEngine.getInstance().releaseProjectionRender(mCastReceiverIdx);
                        bCasting = false;
                        this.mCastReceivers.clear();
                        Log.i(TAG, "stop cast clicked");
                        castButton.setText("投屏");
                    } else {
                        updateCastReceivers();
                    }
                } else {
                    ProjectionEngine.setLicenseID(BaseApp.APP_ID);
                    ProjectionEngine.getInstance().setUpnpDeviceServerListener(getContext(), upnpListenerImp);
                    try {
                        ProjectionEngine.getInstance().searchDeviceBegin();
                    } catch (IllegalStateException e) {
                        Toast.makeText(getContext(), "当前License不支持投屏", Toast.LENGTH_SHORT).show();
                        return;
                    }
                    castButton.setText("搜索中...");
                    bFindingCastReceiver = true;
                    mainThreadHandler.sendEmptyMessageDelayed(MSG_CANCEL_CAST_SEARCH, 15 * 1000);
                }
                break;
            default:
                break;
        }
    }

    public void clearViewContent() {
        currentPositionInMilliSeconds = 0L;
        positionView.setText("00:00");
        durationView.setText("00:00");
        seekBar.setMax(0);
        seekBar.setProgress(0);
        this.availableResolution = null;
    }

    public void setPreListener(View.OnClickListener preListener) {
        this.preListener = preListener;
    }

    public void setNextListener(View.OnClickListener nextListener) {
        this.nextListener = nextListener;
    }

//    public void onNetworkSpeedUpdate(final int arg0) {
// //        Log.d(TAG, "onNetworkSpeedUpdate speed=" + arg0);
//        // 右移10位，即：除以1024。因为arg0的单位为 Byte per second
//        this.mainThreadHandler.post(new Runnable() {
//
//            @Override
//            public void run() {
//                infoNetworkView.setText((arg0 >> 10) + "KB/s");
//            }
//        });
//
//    }

    public String getDescriptionOfResolution(String resolutionType) {
        String result = "未知";
        try {
            // resolutionType is like 1920x1080,3541000
            // sometimes value is ,232370 if there is no resolution desc in master m3u8 file 
            String[] cuts1 = resolutionType.trim().split(",");
            if (cuts1[0].length() > 0) {
                // cuts1[0] has resultion string
                String[] cuts2 = cuts1[0].trim().split("[xX]");
                if (cuts2.length == 2) {
                    // get the height size
                    int iResult = Integer.parseInt(cuts2[1]);
                    if (iResult <= 0) {
                        result = "未知";
                    } else if (iResult <= 120) {
                        result = "120P";
                    } else if (iResult <= 240) {
                        result = "240P";
                    } else if (iResult <= 360) {
                        result = "360P";
                    } else if (iResult <= 480) {
                        result = "480P";
                    } else if (iResult <= 800) {
                        result = "720P";
                    } else if (iResult <= 1080) {
                        result = "1080P";
                    } else if (iResult <= 2160) {
                        result = "4K";
                    } else {
                        result = "8K";
                    }
                }
            }
        } catch (Exception e) {
            Log.d(TAG, "getDescriptionOfResolution exception:" + e.getMessage());
        }
        Log.d(TAG, "getDescriptionOfResolution orig=" + resolutionType + ";result=" + result);

        return result;
    }

    public void setDownloadListener(View.OnClickListener downloadListener) {
        this.downloadListener = downloadListener;
    }

    public Handler getMainThreadHandler() {
        return mainThreadHandler;
    }

    @Override
    public boolean handleMessage(Message msg) {
        if (msg.what == MSG_CANCEL_CAST_SEARCH) {
            castButton.setText("投屏");
            ProjectionEngine.getInstance().searchDeviceEnd();
            Toast.makeText(getContext(), "未找到设备，请稍候重试", Toast.LENGTH_SHORT).show();
        }
        return false;
    }

    public String[] getAvailableResolution() {
        return availableResolution;
    }

    public void setAvailableResolution(String[] fetchResolution) {
        Log.d(TAG, "setAvailableResolution = " + Arrays.toString(fetchResolution));
        if (fetchResolution != null && fetchResolution.length > 1) {
            String[] availableResolutionDesc = new String[fetchResolution.length];
            for (int i = 0; i < fetchResolution.length; ++i) {
                availableResolutionDesc[i] = getDescriptionOfResolution(fetchResolution[i]);
            }
            this.availableResolution = availableResolutionDesc;
        }
    }

    public interface OnCastStateListener {
        int CAST_START = 0;
        int CAST_PAUSED = 1;
        int CAST_END = 2;
        void onCastStateChanged(int state);
    }

    public void setOnCastStateListener(OnCastStateListener castStateListener) {
        mOnCastStateListener = castStateListener;
    }

    private DeviceUpnp.Devicelistener upnpListenerImp = new  DeviceUpnp.Devicelistener() {

        @Override
        public void OnAddDevice(DeviceUpnp device) {
            Log.d(TAG, "onAddDevice " + device.getUuid());
            for (DeviceUpnp deviceUpnp : mCastReceivers) {
                if (deviceUpnp.getUuid().equals(device.getUuid())) {
                    return;
                }
            }
            mCastReceivers.add(device);
            if (bFindingCastReceiver) {
                mainThreadHandler.removeMessages(MSG_CANCEL_CAST_SEARCH);
                updateCastReceivers();
            }
        }

        @Override
        public void OnRemoveDevice(DeviceUpnp device) {

        }
    };

    private ProjectionRenderListenerImp projectionRenderListenerImp = new ProjectionRenderListenerImp();
    class ProjectionRenderListenerImp  implements ProjectionRender.ProjectionRenderListener {
        @Override
        public void onSetAVTransportURI(int renderId, boolean result) {
            Log.i(TAG, "ProjectionRender onSetAVTransportURI " + renderId + ", result: " + result);
        }

        @Override
        public void onPaused(int renderId, boolean result) {
            Log.i(TAG, "ProjectionRender onPaused " + renderId + ", result " + result);
            mCastState = OnCastStateListener.CAST_PAUSED;
            mainThreadHandler.post(new Runnable() {
                @Override
                public void run() {
                    if (mOnCastStateListener != null) {
                        mOnCastStateListener.onCastStateChanged(OnCastStateListener.CAST_PAUSED);
                    }
                }
            });
        }

        public ProjectionRenderListenerImp() {
        }

        @Override
        public void onDuration(int renderId, final Integer duration) {
            // Log.i(TAG, "ProjectionRender onDuration " + renderId + ", duration " + duration);
            mainThreadHandler.post(new Runnable() {
                @Override
                public void run() {
                    setMax(duration * 1000);
                }
            });
        }

        @Override
        public void onCurrent(int renderId, final Integer current) {
            // Log.i(TAG, "ProjectionRender onCurrentPosition " + renderId + ", position " + current);
            mainThreadHandler.post(new Runnable() {
                @Override
                public void run() {
                    seekBar.setProgress(current * 1000);
                }
            });
        }

        @Override
        public void onPlayed(int renderId, boolean result) {
            Log.i(TAG, "ProjectionRender onPlayed " + renderId + ", result " + result);
            mCastState = OnCastStateListener.CAST_START;
            mainThreadHandler.post(new Runnable() {
                @Override
                public void run() {
                    mVideoView.stopPlayback();
                    if (mOnCastStateListener != null) {
                        mOnCastStateListener.onCastStateChanged(OnCastStateListener.CAST_START);
                    }
                    playButton.setBackgroundResource(R.drawable.toggle_btn_pause);
                }
            });
        }

        @Override
        public void onStoped(int renderId, boolean result) {
            Log.i(TAG, "ProjectionRender onStoped " + renderId + ", result " + result);
            mCastState = OnCastStateListener.CAST_END;
            mainThreadHandler.post(new Runnable() {
                @Override
                public void run() {
                    if (mOnCastStateListener != null) {
                        mOnCastStateListener.onCastStateChanged(OnCastStateListener.CAST_END);
                    }
                    playButton.setBackgroundResource(R.drawable.toggle_btn_play);
                }
            });
        }
    }

    private void updateCastReceivers() {
        mainThreadHandler.post(new Runnable() {
            @Override
            public void run() {
                castButton.setText("投屏");
                castButton.requestLayout();
                OnItemClickListener listener = new OnItemClickListener() {
                    @Override
                    public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                        bFindingCastReceiver = false;
                        DeviceUpnp device = mCastReceivers.get(position);
                        if (null != device) {
                            mCastReceiverIdx =
                                    ProjectionEngine.getInstance()
                                            .distributeProjectionRender(device, projectionRenderListenerImp);
                            ProjectionEngine.getInstance().setAVTransportURL(mCastReceiverIdx,
                                    mVideoView.getCurrentPlayingUrl());
                            ProjectionEngine.getInstance().play(mCastReceiverIdx, 1);
                            bCasting = true;
                            Log.i(TAG, "start cast clicked");
                            castButton.setText("结束投屏");
                        }
                    }

                };
                ArrayList<String> receiverNames = new ArrayList<>(mCastReceivers.size());
                for (DeviceUpnp deviceUpnp : mCastReceivers) {
                    receiverNames.add(deviceUpnp.getFriendlName());
                }
                // popup window
                if (castPopWindow != null && castPopWindow.isShowing()) {
                    castPopWindow.dismiss();
                }
                castPopWindow = DefaultPopWindow.createOptionListPopWindow((Activity) getContext(),
                                    receiverNames.toArray(new String[0]), 120, listener);
                int[] l = new int[2];
                castButton.getLocationOnScreen(l);
                castPopWindow.showPopupWindowAt(((Activity) getContext()).findViewById(R.id.root)
                        , Gravity.NO_GRAVITY, l[0] + mCastButtonWidth, l[1] - 50);
            }
        });
    }
}
