package com.baidu.cloud.videoplayer.demo.danmu;

import static java.nio.charset.StandardCharsets.UTF_8;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.baidu.cloud.videoplayer.demo.helper.CommonUtils;

import android.annotation.SuppressLint;
import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.Path;
import android.graphics.PorterDuff;
import android.graphics.PorterDuffXfermode;
import android.graphics.RectF;
import android.os.Build;
import android.os.Handler;
import android.os.HandlerThread;
import android.os.Looper;
import android.text.TextUtils;
import android.util.AttributeSet;
import android.view.MotionEvent;
import android.view.View;
import androidx.core.graphics.PathParser;
import master.flame.danmaku.controller.DrawHandler;
import master.flame.danmaku.controller.DrawHelper;
import master.flame.danmaku.controller.IDanmakuView;
import master.flame.danmaku.controller.IDanmakuViewController;
import master.flame.danmaku.danmaku.model.BaseDanmaku;
import master.flame.danmaku.danmaku.model.IDanmakus;
import master.flame.danmaku.danmaku.model.android.DanmakuContext;
import master.flame.danmaku.danmaku.parser.BaseDanmakuParser;
import master.flame.danmaku.danmaku.renderer.IRenderer;
import master.flame.danmaku.danmaku.util.SystemClock;

public class BaiduDanmakuView extends View implements IDanmakuView, IDanmakuViewController {

    public static final String TAG = "BaiduDanmakuView";

    private DrawHandler.Callback mCallback;

    private HandlerThread mHandlerThread;

    protected volatile DrawHandler handler;

    private boolean isSurfaceCreated;

    private boolean mEnableDanmakuDrwaingCache = true;

    private OnDanmakuClickListener mOnDanmakuClickListener;

    private float mXOff;

    private float mYOff;

    private OnClickListener mOnClickListener;

    private CustomDanmaTouchHelper mTouchHelper;

    private boolean mShowFps;

    private boolean mDanmakuVisible = true;

    protected int mDrawingThreadType = THREAD_TYPE_NORMAL_PRIORITY;

    private Object mDrawMonitor = new Object();

    private boolean mDrawFinished = false;

    protected boolean mRequestRender = false;

    private long mUiThreadId;

    /**
     * 智能弹幕相关
     */
    private static final String NODE_PATH_START = "d";
    private static final String NODE_WIDTH = "width";
    private static final String NODE_HEIGHT = "height";
    private static final String TRANSFORM = "transform";
    private static final String SCALE = "scale";
    private static final String TRANSLATE = "translate";
    private static final Matrix MATRIX = new Matrix();

    private String mSvgMaskData;
    private RectF mVideoViewRecf;
    private int[] locationInScreen;
    private Matrix mSvgMaskMatrix;
    private Paint mSvgMaskPaint;
    private PorterDuffXfermode mSvgMode;

    public BaiduDanmakuView(Context context) {
        super(context);
        init();
    }

    public BaiduDanmakuView(Context context, AttributeSet attrs) {
        super(context, attrs);
        init();
    }

    public BaiduDanmakuView(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
        init();
    }

    private void init() {
        mUiThreadId = Thread.currentThread().getId();
        setBackgroundColor(Color.TRANSPARENT);
        setDrawingCacheBackgroundColor(Color.TRANSPARENT);
        DrawHelper.useDrawColorToClearCanvas(true, false);
        mTouchHelper = CustomDanmaTouchHelper.instance(this);
        locationInScreen = new int[2];
        mSvgMaskMatrix = new Matrix();
        mSvgMaskPaint = new Paint();
        mSvgMode = new PorterDuffXfermode(PorterDuff.Mode.DST_OUT);
    }

    public void addDanmaku(BaseDanmaku item) {
        if (handler != null) {
            handler.addDanmaku(item);
        }
    }

    @Override
    public void invalidateDanmaku(BaseDanmaku item, boolean remeasure) {
        if (handler != null) {
            handler.invalidateDanmaku(item, remeasure);
        }
    }

    @Override
    public void removeAllDanmakus(boolean isClearDanmakusOnScreen) {
        if (handler != null) {
            handler.removeAllDanmakus(isClearDanmakusOnScreen);
        }
    }

    @Override
    public void removeAllLiveDanmakus() {
        if (handler != null) {
            handler.removeAllLiveDanmakus();
        }
    }

    @Override
    public IDanmakus getCurrentVisibleDanmakus() {
        if (handler != null) {
            return handler.getCurrentVisibleDanmakus();
        }

        return null;
    }

    public void setCallback(DrawHandler.Callback callback) {
        mCallback = callback;
        if (handler != null) {
            handler.setCallback(callback);
        }
    }

    public void setSvgMaskData(String svgMaskData) {
        mSvgMaskData = svgMaskData;
    }

    public void setVideoViewRecf(RectF recf) {
        mVideoViewRecf = recf;
    }

    @Override
    public void release() {
        stop();
        if (mDrawTimes != null) {
            mDrawTimes.clear();
        }
        if (mSvgMaskMatrix != null) {
            mSvgMaskMatrix.reset();
        }
    }

    @Override
    public void stop() {
        stopDraw();
    }

    private synchronized void stopDraw() {
        if (this.handler == null) {
            return;
        }
        DrawHandler handler = this.handler;
        this.handler = null;
        unlockCanvasAndPost();
        if (handler != null) {
            handler.quit();
        }
        HandlerThread handlerThread = this.mHandlerThread;
        mHandlerThread = null;
        if (handlerThread != null) {
            try {
                handlerThread.join();
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            handlerThread.quit();
        }
    }

    protected synchronized Looper getLooper(int type) {
        if (mHandlerThread != null) {
            mHandlerThread.quit();
            mHandlerThread = null;
        }

        int priority;
        switch (type) {
            case THREAD_TYPE_MAIN_THREAD:
                return Looper.getMainLooper();
            case THREAD_TYPE_HIGH_PRIORITY:
                priority = android.os.Process.THREAD_PRIORITY_URGENT_DISPLAY;
                break;
            case THREAD_TYPE_LOW_PRIORITY:
                priority = android.os.Process.THREAD_PRIORITY_LOWEST;
                break;
            case THREAD_TYPE_NORMAL_PRIORITY:
            default:
                priority = android.os.Process.THREAD_PRIORITY_DEFAULT;
                break;
        }
        String threadName = "DFM Handler Thread #" + priority;
        mHandlerThread = new HandlerThread(threadName, priority);
        mHandlerThread.start();
        return mHandlerThread.getLooper();
    }

    private void prepare() {
        if (handler == null) {
            handler = new DrawHandler(getLooper(mDrawingThreadType), this, mDanmakuVisible);
        }
    }

    @Override
    public void prepare(BaseDanmakuParser parser, DanmakuContext config) {
        prepare();
        handler.setConfig(config);
        handler.setParser(parser);
        handler.setCallback(mCallback);
        handler.prepare();
    }

    @Override
    public boolean isPrepared() {
        return handler != null && handler.isPrepared();
    }

    @Override
    public DanmakuContext getConfig() {
        if (handler == null) {
            return null;
        }
        return handler.getConfig();
    }

    @Override
    public void showFPS(boolean show){
        mShowFps = show;
    }
    private static final int MAX_RECORD_SIZE = 50;
    private static final int ONE_SECOND = 1000;
    private LinkedList<Long> mDrawTimes;

    protected boolean mClearFlag;
    private float fps() {
        long lastTime = SystemClock.uptimeMillis();
        mDrawTimes.addLast(lastTime);
        Long first = mDrawTimes.peekFirst();
        if (first == null) {
            return 0.0f;
        }
        float dtime = lastTime - first;
        int frames = mDrawTimes.size();
        if (frames > MAX_RECORD_SIZE) {
            mDrawTimes.removeFirst();
        }
        return dtime > 0 ? mDrawTimes.size() * ONE_SECOND / dtime : 0.0f;
    }
    @Override
    public long drawDanmakus() {
        if (!isSurfaceCreated) {
            return 0;
        }
        if (!isShown()) {
            return -1;
        }
        long stime = SystemClock.uptimeMillis();
        lockCanvas();
        return SystemClock.uptimeMillis() - stime;
    }

    @SuppressLint("NewApi")
    private void postInvalidateCompat() {
        mRequestRender = true;
        if (Build.VERSION.SDK_INT >= 16) {
            this.postInvalidateOnAnimation();
        } else {
            this.postInvalidate();
        }
    }

    protected void lockCanvas() {
        if (mDanmakuVisible == false) {
            return;
        }
        postInvalidateCompat();
        synchronized (mDrawMonitor) {
            while ((!mDrawFinished) && (handler != null)) {
                try {
                    mDrawMonitor.wait(200);
                } catch (InterruptedException e) {
                    if (mDanmakuVisible == false || handler == null || handler.isStop()) {
                        break;
                    } else {
                        Thread.currentThread().interrupt();
                    }
                }
            }
            mDrawFinished = false;
        }
    }

    private void lockCanvasAndClear() {
        mClearFlag = true;
        lockCanvas();
    }

    private void unlockCanvasAndPost() {
        synchronized (mDrawMonitor) {
            mDrawFinished = true;
            mDrawMonitor.notifyAll();
        }
    }

    @Override
    protected void onDraw(Canvas canvas) {
        if ((!mDanmakuVisible) && (!mRequestRender)) {
            super.onDraw(canvas);
            return;
        }
        if (mClearFlag) {
            DrawHelper.clearCanvas(canvas);
            mClearFlag = false;
        } else {
            if (handler != null) {
                int layer = -1;
                if (!TextUtils.isEmpty(mSvgMaskData) && mVideoViewRecf != null) {
                    layer = canvas.saveLayer(0, 0, getViewWidth(), getViewHeight(), null,
                            Canvas.ALL_SAVE_FLAG);
                }

                IRenderer.RenderingState rs = handler.draw(canvas);
                if (mShowFps) {
                    if (mDrawTimes == null) {
                        mDrawTimes = new LinkedList<Long>();
                    }
                    String fps = String.format(Locale.getDefault(),
                            "fps %.2f,time:%d s,cache:%d,miss:%d", fps(), getCurrentTime() / 1000,
                            rs.cacheHitCount, rs.cacheMissCount);
                    DrawHelper.drawFPS(canvas, fps);
                }

                if (!TextUtils.isEmpty(mSvgMaskData) && mVideoViewRecf != null) {
                    // 使用 PorterDuff.Mode.DST_OUT 绘制遮罩 Path
                    mSvgMaskPaint.setColor(Color.WHITE);
                    mSvgMaskPaint.setXfermode(mSvgMode);

                    Path path = parseSvgForPath(mSvgMaskData, mVideoViewRecf,
                            CommonUtils.getScreenWidth(getContext(), false),
                            CommonUtils.getScreenHeight(getContext(), false));

                    // 考虑弹幕view位置引入的偏移量
                    getView().getLocationOnScreen(locationInScreen);
                    mSvgMaskMatrix.reset();
                    mSvgMaskMatrix.postScale(1, 1);
                    mSvgMaskMatrix.postTranslate(-locationInScreen[0], -locationInScreen[1]);
                    path.transform(mSvgMaskMatrix);

                    canvas.drawPath(path, mSvgMaskPaint);
                    mSvgMaskPaint.setXfermode(null);
                    canvas.restoreToCount(layer);
                }
            }
        }
        mRequestRender = false;
        unlockCanvasAndPost();
    }

    @Override
    protected void onLayout(boolean changed, int left, int top, int right, int bottom) {
        super.onLayout(changed, left, top, right, bottom);
        if (handler != null) {
            handler.notifyDispSizeChanged(right - left, bottom - top);
        }
        isSurfaceCreated = true;
    }

    public void toggle() {
        if (isSurfaceCreated) {
            if (handler == null) {
                start();
            } else if (handler.isStop()) {
                resume();
            } else {
                pause();
            }
        }
    }

    @Override
    public void pause() {
        if (handler != null) {
            handler.removeCallbacks(mResumeRunnable);
            handler.pause();
        }
    }

    private int mResumeTryCount = 0;

    private Runnable mResumeRunnable = new Runnable() {
        @Override
        public void run() {
            DrawHandler drawHandler = handler;
            if (drawHandler == null) {
                return;
            }
            mResumeTryCount++;
            if (mResumeTryCount > 4 || BaiduDanmakuView.super.isShown()) {
                drawHandler.resume();
            } else {
                drawHandler.postDelayed(this, 100 * mResumeTryCount);
            }
        }
    };

    @Override
    public void resume() {
        if (handler != null && handler.isPrepared()) {
            mResumeTryCount = 0;
            handler.post(mResumeRunnable);
        }  else if (handler == null) {
            restart();
        }
    }

    @Override
    public boolean isPaused() {
        if (handler != null) {
            return handler.isStop();
        }
        return false;
    }

    public void restart() {
        stop();
        start();
    }

    @Override
    public void start() {
        start(0);
    }

    @Override
    public void start(long position) {
        Handler handler = this.handler;
        if (handler == null) {
            prepare();
            handler = this.handler;
        } else {
            handler.removeCallbacksAndMessages(null);
        }
        if (handler != null) {
            handler.obtainMessage(DrawHandler.START, position).sendToTarget();
        }
    }

    @Override
    public boolean onTouchEvent(MotionEvent event) {
        int size = mTouchHelper.touchHitDanmaku(event.getX(), event.getY()).getCollection().size();
        boolean isEventConsumed = mTouchHelper.onTouchEvent(event)
                && size != 0;
        if (!isEventConsumed) {
            super.onTouchEvent(event);
            return false;
        }
        return isEventConsumed;
    }

    public void seekTo(Long ms) {
        if (handler != null) {
            handler.seekTo(ms);
        }
    }


    public void enableDanmakuDrawingCache(boolean enable) {
        mEnableDanmakuDrwaingCache = enable;
    }

    @Override
    public boolean isDanmakuDrawingCacheEnabled() {
        return mEnableDanmakuDrwaingCache;
    }

    @Override
    public boolean isViewReady() {
        return isSurfaceCreated;
    }

    @Override
    public int getViewWidth() {
        return super.getWidth();
    }

    @Override
    public int getViewHeight() {
        return super.getHeight();
    }

    @Override
    public View getView() {
        return this;
    }

    @Override
    public void show() {
        showAndResumeDrawTask(null);
    }

    @Override
    public void showAndResumeDrawTask(Long position) {
        mDanmakuVisible = true;
        mClearFlag = false;
        if (handler == null) {
            return;
        }
        handler.showDanmakus(position);
    }

    @Override
    public void hide() {
        mDanmakuVisible = false;
        if (handler == null) {
            return;
        }
        handler.hideDanmakus(false);
    }

    @Override
    public long hideAndPauseDrawTask() {
        mDanmakuVisible = false;
        if (handler == null) {
            return 0;
        }
        return handler.hideDanmakus(true);
    }

    @Override
    public void clear() {
        if (!isViewReady()) {
            return;
        }
        if (!mDanmakuVisible || Thread.currentThread().getId() == mUiThreadId) {
            mClearFlag = true;
            postInvalidateCompat();
        } else {
            lockCanvasAndClear();
        }
    }

    @Override
    public boolean isShown() {
        return mDanmakuVisible && super.isShown();
    }

    @Override
    public void setDrawingThreadType(int type) {
        mDrawingThreadType  = type;
    }

    @Override
    public long getCurrentTime() {
        if (handler != null) {
            return handler.getCurrentTime();
        }
        return 0;
    }

    @Override
    @SuppressLint("NewApi")
    public boolean isHardwareAccelerated() {
        // >= 3.0
        if (Build.VERSION.SDK_INT >= 11) {
            return super.isHardwareAccelerated();
        } else {
            return false;
        }
    }

    @Override
    public void clearDanmakusOnScreen() {
        if (handler != null) {
            handler.clearDanmakusOnScreen();
        }
    }

    @Override
    public void setOnDanmakuClickListener(OnDanmakuClickListener listener) {
        mOnDanmakuClickListener = listener;
    }

    @Override
    public void setOnDanmakuClickListener(OnDanmakuClickListener listener, float xOff, float yOff) {
        mOnDanmakuClickListener = listener;
        mXOff = xOff;
        mYOff = yOff;
    }

    @Override
    public OnDanmakuClickListener getOnDanmakuClickListener() {
        return mOnDanmakuClickListener;
    }

    @Override
    public float getXOff() {
        return mXOff;
    }

    @Override
    public float getYOff() {
        return mYOff;
    }

    public void forceRender() {
        mRequestRender = true;
        handler.forceRender();
    }

    private static Path parseSvgForPath(String svgString, RectF videoRectF, int screenWidth, int screenHeight) {
        if (TextUtils.isEmpty(svgString)) {
            return new Path();
        }
        // 外框path
        Path outerFramePath = new Path();

        float videoViewWidth = videoRectF.width();
        float videoViewHeight = videoRectF.height();
        float videoLeftPosition = videoRectF.left;
        float videoTopPosition = videoRectF.top;

        InputStream inputStream = new ByteArrayInputStream(svgString.getBytes(UTF_8));
        try {
            final Document document = DocumentBuilderFactory.newInstance()
                    .newDocumentBuilder()
                    .parse(inputStream);
            final String xpathExpression = "//@*";
            final XPath xPath = XPathFactory.newInstance().newXPath();
            final XPathExpression expression = xPath.compile(xpathExpression);
            final NodeList svgPaths = (NodeList) expression.evaluate(document,
                    XPathConstants.NODESET);

            int maskWidth = 1;
            int maskHeight = 1;
            String transform = "";
            List<Path> pathList = new ArrayList<>();
            for (int i = 0; i < svgPaths.getLength(); i++) {
                final Node node = svgPaths.item(i);
                switch (node.getNodeName()) {
                    case NODE_PATH_START:
                        final Path path = PathParser.createPathFromPathData(node.getTextContent());
                        pathList.add(path);
                        break;
                    case NODE_WIDTH:
                        if (node.getTextContent().contains("pt")) {
                            if (node.getTextContent().contains(".")) {
                                maskWidth = (int) Float.parseFloat(node.getTextContent().replace("pt", ""));
                            } else {
                                maskWidth =
                                        Integer.parseInt(node.getTextContent().replace("pt", ""));
                            }
                        } else {
                            if (node.getTextContent().contains(".")) {
                                maskWidth = (int) Float.parseFloat(node.getTextContent().replace(
                                        "px", ""));
                            } else {
                                maskWidth =
                                        Integer.parseInt(node.getTextContent().replace("px", ""));
                            }
                        }
                        break;
                    case NODE_HEIGHT:
                        if (node.getTextContent().contains("pt")) {
                            if (node.getTextContent().contains(".")) {
                                maskHeight = (int) Float.parseFloat(node.getTextContent().replace("pt", ""));
                            } else {
                                maskHeight =
                                        Integer.parseInt(node.getTextContent().replace("pt", ""));
                            }
                        } else {
                            if (node.getTextContent().contains(".")) {
                                maskHeight = (int) Float.parseFloat(node.getTextContent().replace(
                                        "px", ""));
                            } else {
                                maskHeight =
                                        Integer.parseInt(node.getTextContent().replace("px", ""));
                            }
                        }
                        break;
                    case TRANSFORM:
                        transform = node.getTextContent();
                        break;
                }
            }

            float svgScaleX = 1;
            float svgScaleY = 1;
            float svgTransX = 0;
            float svgTransY = 0;
            if (!TextUtils.isEmpty(transform)) {
                Point scale = getTransform(transform, SCALE);
                if (scale != null) {
                    svgScaleX = scale.x;
                    svgScaleY = scale.y;
                }
                Point translate = getTransform(transform, TRANSLATE);
                if (translate != null) {
                    svgTransX = translate.x;
                    svgTransY = translate.y;
                }
            }

            if (pathList.size() > 0) {
                RectF outerRectF = new RectF(videoRectF);
                outerRectF.left = Math.max(outerRectF.left, 0);
                outerRectF.top = Math.max(outerRectF.top, 0);

                if (screenHeight != 0 && screenWidth != 0) {
                    outerRectF.right = Math.min(screenWidth, videoRectF.right);
                    outerRectF.bottom = Math.min(screenHeight, videoRectF.bottom);
                }
                outerFramePath.addRect(outerRectF, Path.Direction.CW);
            }

            // 内部遮罩path
            for (final Path onePath : pathList) {
                MATRIX.reset();
                MATRIX.postScale(svgScaleX, svgScaleY);
                MATRIX.postTranslate(svgTransX, svgTransY);
                final float scaleX = videoViewWidth * 1.0f / maskWidth;
                final float scaleY = videoViewHeight * 1.0f / maskHeight;
                MATRIX.postScale(scaleX, scaleY);
                MATRIX.postTranslate(videoLeftPosition, videoTopPosition);
                Path path = new Path(onePath);
                path.transform(MATRIX);

                if (Build.VERSION.SDK_INT >= 21) {
                    outerFramePath.op(path, Path.Op.DIFFERENCE);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                inputStream.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        return outerFramePath;
    }

    private static Point getTransform(final String transformStr, final String fun) {
        int index = transformStr.indexOf(fun);
        if (index == -1) {
            return null;
        }

        String valueStr = transformStr.substring(index + fun.length());
        String regex = "\\((\\-|\\+)?\\d+(\\.\\d+)?,(\\-|\\+)?\\d+(\\.\\d+)?\\)";
        Pattern compile = Pattern.compile(regex);
        Matcher matcher = compile.matcher(valueStr);
        if (!matcher.lookingAt()) {
            return null;
        }

        int end = matcher.end();
        String[] result = valueStr.substring(0, end).split(",");
        if (result.length != 2) {
            return null;
        }
        String x = result[0].substring(1);
        String y = result[1].substring(0, result[1].length() - 1);
        return new Point( Float.parseFloat(x), Float.parseFloat(y));
    }

    private static class Point {
        private float x;
        private float y;

        public Point(final float x, final float y) {
            this.x = x;
            this.y = y;
        }
    }
}

