package com.baidu.cloud.videoplayer.demo.danmu;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Rect;
import android.graphics.RectF;
import android.graphics.drawable.Drawable;
import android.text.Spannable;
import android.text.SpannableStringBuilder;
import android.text.TextPaint;
import android.text.style.ImageSpan;
import android.util.Log;

import androidx.annotation.NonNull;

import com.baidu.cloud.videoplayer.demo.C;
import com.baidu.cloud.videoplayer.demo.helper.CommonUtils;

import java.io.InputStream;
import java.util.HashMap;

import master.flame.danmaku.controller.DrawHandler;
import master.flame.danmaku.controller.IDanmakuView;
import master.flame.danmaku.danmaku.loader.ILoader;
import master.flame.danmaku.danmaku.loader.IllegalDataException;
import master.flame.danmaku.danmaku.loader.android.DanmakuLoaderFactory;
import master.flame.danmaku.danmaku.model.BaseDanmaku;
import master.flame.danmaku.danmaku.model.DanmakuTimer;
import master.flame.danmaku.danmaku.model.IDanmakus;
import master.flame.danmaku.danmaku.model.IDisplayer;
import master.flame.danmaku.danmaku.model.android.DanmakuContext;
import master.flame.danmaku.danmaku.model.android.Danmakus;
import master.flame.danmaku.danmaku.model.android.SpannedCacheStuffer;
import master.flame.danmaku.danmaku.parser.BaseDanmakuParser;
import master.flame.danmaku.danmaku.parser.IDataSource;

public class DefaultDanmuView extends BaiduDanmakuView {

    private static final String TAG = "DefaultDanmuView";
    private DanmakuContext mContext;
    private BaseDanmakuParser mParser;
    private long mInitStartPosMs = 0L;

    private static final float DEFAULT_SPEED = 1.2f;

    public DefaultDanmuView(@NonNull Context context, @NonNull InputStream inputStream) {
        super(context);
        mParser = createParser(context, inputStream);
        initDanmuView();
    }

    private void initDanmuView() {
        // 设置最大显示行数
        HashMap<Integer, Integer> maxLinesPair = new HashMap<>();
        maxLinesPair.put(BaseDanmaku.TYPE_SCROLL_RL, 30); // 滚动弹幕最大显示行数
        // 设置是否禁止重叠
        HashMap<Integer, Boolean> overlappingEnablePair = new HashMap<>();
        overlappingEnablePair.put(BaseDanmaku.TYPE_SCROLL_RL, true);
        overlappingEnablePair.put(BaseDanmaku.TYPE_FIX_TOP, true);

        mContext = DanmakuContext.create();
        mContext.setDanmakuStyle(IDisplayer.DANMAKU_STYLE_STROKEN, 3)
                .setDuplicateMergingEnabled(false)
                .setScrollSpeedFactor(DEFAULT_SPEED)
                .setScaleTextSize(1.2f)
                // 控制是否应用行数限制
                // .setMaximumLines(maxLinesPair)
                // 控制是否应用重叠设置
                // .preventOverlapping(overlappingEnablePair)
                .setDanmakuMargin(40)
                .setCacheStuffer(new BackgroundCacheStuffer(), null);
        setCallback(new DrawHandler.Callback() {
            @Override
            public void prepared() {
                if (mInitStartPosMs > 0) {
                    seekTo(mInitStartPosMs);
                }
                start();
            }

            @Override
            public void updateTimer(DanmakuTimer timer) {

            }

            @Override
            public void danmakuShown(BaseDanmaku danmaku) {

            }

            @Override
            public void drawingFinished() {

            }
        });
        enableDanmakuDrawingCache(true);
    }

    public void prepare() {
        prepare(0);
    }

    public void prepare(long initStartPosMs) {
        if (isPrepared()) {
            restart();
        }
        mInitStartPosMs = initStartPosMs;
        setOnDanmakuClickListener(new IDanmakuView.OnDanmakuClickListener() {

            @Override
            public boolean onDanmakuClick(IDanmakus danmakus) {
                BaseDanmaku latest = danmakus.last();
                if (null != latest) {
                    if (latest.tag == null) {
                        latest.text = "+1" + latest.text;
                        latest.textColor = Color.GREEN;
                        latest.tag = true; // 代表被点击过了
                    } else {
                        latest.text = latest.text.subSequence(2, latest.text.length());
                        latest.textColor = C.WHITE_80;
                        latest.tag = null;
                    }
                    invalidateDanmaku(latest, true);
                    Log.d(TAG, "onDanmakuClick: text of latest danmaku:" + latest.text);
                    return true;
                }
                return false;
            }

            @Override
            public boolean onDanmakuLongClick(IDanmakus danmakus) {
                return false;
            }

            @Override
            public boolean onViewClick(IDanmakuView view) {
                return false;
            }
        });
        prepare(mParser, mContext);
    }

    @Override
    public void start() {
        if (isPrepared() && isPaused()) {
            super.resume();
        }
    }

    @Override
    public void pause() {
        if (isPrepared()) {
            super.pause();
        }
    }

    public void seekTo(long millisec) {
        if (isPrepared()) {
            super.seekTo(millisec);
        }
    }

    @Override
    public void clear() {
        super.clear();
        super.clearDanmakusOnScreen();
    }

    @Override
    public void release() {
        super.release();
    }

    @Override
    public void show() {
        super.show();
    }

    @Override
    public void hide() {
        super.hide();
    }

    public boolean isVisible() {
        return super.isShown();
    }

    public void setSpeed(float speed) {
        if (speed > 0) {
            mContext.setScrollSpeedFactor(DEFAULT_SPEED / speed);
        }
    }

    public void addDanmuItem(String text, boolean isManual) {
        BaseDanmaku danmaku = mContext.mDanmakuFactory.createDanmaku(BaseDanmaku.TYPE_SCROLL_RL);
        if (danmaku == null) {
            return;
        }

        danmaku.text = text;
        danmaku.padding = 5;
        // 手动发送的弹幕赋予高优先级，反之赋予低优先级，可能会被各种过滤器过滤并隐藏显示
        danmaku.priority = isManual ? (byte) 1 : 0;
        danmaku.isLive = false;
        danmaku.setTime(getCurrentTime() + 1200);
        danmaku.textSize = CommonUtils.sp2px(getContext(), 12);
        danmaku.textColor = C.WHITE_80;
        danmaku.textShadowColor = Color.GRAY;
        danmaku.borderColor = isManual ? Color.GREEN : Color.TRANSPARENT;
        addDanmaku(danmaku);
    }

    public void addDanmuItemWithDrawable(String text, Drawable drawable, boolean isManual) {
        BaseDanmaku danmaku = mContext.mDanmakuFactory.createDanmaku(BaseDanmaku.TYPE_SCROLL_RL);
        if (danmaku == null) {
            return;
        }

        danmaku.text = createSpannable(text, drawable);
        danmaku.priority = isManual ? (byte) 1 : 0;
        danmaku.isLive = false;
        danmaku.setTime(getCurrentTime() + 1200);
        danmaku.textSize = CommonUtils.sp2px(getContext(), 12);
        danmaku.textColor = Color.RED;
        danmaku.textShadowColor = 0;
        addDanmaku(danmaku);
    }

    private SpannableStringBuilder createSpannable(String text, Drawable drawable) {
        String text1 = "placeholder";
        SpannableStringBuilder spannableStringBuilder = new SpannableStringBuilder(text1);
        int size = CommonUtils.dp2px(getContext(), 20);
        drawable.setBounds(0, 0, size, size);
        CenteredImageSpan span = new CenteredImageSpan(drawable);
        spannableStringBuilder.setSpan(span, 0, text1.length(), Spannable.SPAN_INCLUSIVE_EXCLUSIVE);
        spannableStringBuilder.append(text);
        return spannableStringBuilder;
    }

    public class CenteredImageSpan extends ImageSpan {

        public CenteredImageSpan(final Drawable drawable) {
            super(drawable);
        }

        @Override
        public int getSize(Paint paint, CharSequence text, int start, int end,
                           Paint.FontMetricsInt fm) {
            Drawable d = getDrawable();
            Rect rect = d.getBounds();
            if (fm != null) {
                Paint.FontMetricsInt fmPaint = paint.getFontMetricsInt();
                int fontHeight = fmPaint.bottom - fmPaint.top;
                int drHeight = rect.bottom - rect.top;

                int top = drHeight / 2 - fontHeight / 4;
                int bottom = drHeight / 2 + fontHeight / 4;

                fm.ascent = -bottom;
                fm.top = -bottom;
                fm.bottom = top;
                fm.descent = top;
            }
            return rect.right;
        }

        @Override
        public void draw(Canvas canvas, CharSequence text, int start, int end,
                         float x, int top, int y, int bottom, Paint paint) {
            Drawable b = getDrawable();
            canvas.save();
            int transY;
            transY = ((bottom - top) - b.getBounds().bottom) / 2 + top;
            canvas.translate(x, transY);
            b.draw(canvas);
            canvas.restore();
        }
    }


    private BaseDanmakuParser createParser(Context context, InputStream stream) {
        if (stream == null) {
            return new BaseDanmakuParser() {

                @Override
                protected Danmakus parse() {
                    return new Danmakus();
                }
            };
        }

        ILoader loader = DanmakuLoaderFactory.create(DanmakuLoaderFactory.TAG_BILI);

        try {
            loader.load(stream);
        } catch (IllegalDataException e) {
            e.printStackTrace();
        }
        BaseDanmakuParser parser = new DefaultDanmuParser(context);
        IDataSource<?> dataSource = loader.getDataSource();
        parser.load(dataSource);
        return parser;
    }

    private class BackgroundCacheStuffer extends SpannedCacheStuffer {
        // 通过扩展SimpleTextCacheStuffer或SpannedCacheStuffer个性化你的弹幕样式
        final Paint paint = new Paint();

        @Override
        public void measure(BaseDanmaku danmaku, TextPaint paint, boolean fromWorkerThread) {
            super.measure(danmaku, paint, fromWorkerThread);
        }

        @Override
        public void drawBackground(BaseDanmaku danmaku, Canvas canvas, float left, float top) {
            paint.setAntiAlias(true);
            paint.setColor(Color.TRANSPARENT); // 黑色 普通
            int radius = CommonUtils.dp2px(getContext(), 10);
            canvas.drawRoundRect(new RectF(left, top, left + danmaku.paintWidth,
                    top + danmaku.paintHeight), radius, radius, paint);
        }

        @Override
        public void drawStroke(BaseDanmaku danmaku, String lineText,
                               Canvas canvas, float left, float top, Paint paint) {
            // 禁用描边绘制
        }
    }
}
