package com.baidu.cloud.videoplayer.demo.helper;

import android.content.Context;
import android.content.res.AssetManager;
import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Point;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Build;
import android.util.Log;
import android.util.TypedValue;
import android.view.Display;
import android.view.KeyCharacterMap;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.ViewConfiguration;
import android.view.WindowManager;

import com.baidu.cloud.media.player.IMediaPlayer;
import com.baidu.cloud.media.player.feature.FeatureManager;
import com.baidu.cloud.videoplayer.demo.BaseApp;
import com.baidu.cloud.videoplayer.demo.C;
import com.baidu.cloud.videoplayer.demo.info.SharedPrefsStore;

import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.Locale;

public class CommonUtils {
    private static final String TAG = "CommonUtils";

    public static boolean getSystemProperty(String key, boolean def) {
        try {
            final ClassLoader cl = ClassLoader.getSystemClassLoader();
            final Class<?> systemProperties = cl.loadClass("android.os.SystemProperties");
            final Class<?>[] paramTypes = new Class[] { String.class, Boolean.TYPE };
            final Method get = systemProperties.getMethod("getBoolean", paramTypes);
            final Object[] params = new Object[] { key, def };
            return (boolean) get.invoke(systemProperties, params);
        } catch (Exception e) {
            e.printStackTrace();
            return def;
        }
    }

    /**
     * Find specific class field
     * @param obj :    class object
     * @param fName : field name
     * @return : specific field
     */
    public static Field getDeclaredField(Object obj, String fName){
        Field field = null;

        if (obj == null) {
            return null;
        }
        try {
            Class cls = obj.getClass();

            /* Loop to upper class, and get specific member field */
            for (; cls != Object.class; cls = cls.getSuperclass()) {
                try {
                    field = cls.getDeclaredField(fName);
                    break;
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
            if (field != null) {
                /* If it is private authority, modify access limit */
                if (!field.isAccessible()) field.setAccessible(true);
            }
        } catch (Exception e) {
            Log.e(TAG, "get " + fName + " field failed: " + e);
        } finally {
            if (field == null) {
                Log.e(TAG, "not found " + fName + " field");
            }
        }
        return field;
    }

    /**
     * Get specific class field value
     * @param obj :    class object
     * @param field : field object
     * @return : specific field value
     */
    public static Object getFieldValue(Object obj, Field field) {
        Object retObject = null;

        if (field != null) {
            try {
                retObject = field.get(obj);
            } catch (Exception e) {
                Log.e(TAG, "get field failed: " + e);
            }
        } else if (obj != null) {
            Log.e(TAG, obj.getClass() + " field is null");
        }
        return retObject;
    }

    /**
     * Invoke static class member method, find method, and call method
     * @param cName : class name
     * @param mName : method name
     * @param types : type array of parameters
     * @param params: value array of parameters
     * @return : method return value
     */
    public static Object invokeStaticMethod(
            String cName, String mName, Class[] types, Object[] params) {
        Object retObject = null;

        try {
            Class<?> cls = Class.forName(cName);
            Method meth = null;

            /* Loop to upper class, and get specific member method */
            for (; cls != Object.class; cls = cls.getSuperclass()) {
                try {
                    meth = cls.getMethod(mName, types);
                    break;
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
            if (meth != null) {
                /* If it is private authority, modify access limit */
                if (!meth.isAccessible()) meth.setAccessible(true);
                retObject = meth.invoke(null, params);
            } else {
                Log.e(TAG, "not found " + mName + " method");
            }
        } catch (Exception e) {
            Log.e(TAG, "invoke " + mName + " method failed: " + e);
        }
        return retObject;
    }

    /**
     * New class instance with params constructor method
     * @param cName : class name
     * @param types : type array of parameters
     * @param params: value array of parameters
     * @return : class object
     */
    public static Object Load(String cName, Class[] types, Object[] params) {
        Object retObject = null;
        try {
            Class<?> cls = Class.forName(cName);

            Constructor ct = cls.getDeclaredConstructor(types);
            /* If it is private authority, modify access limit */
            if (!ct.isAccessible()) {
                ct.setAccessible(true);
            }
            retObject = ct.newInstance(params);
        } catch (Exception e) {
            Log.e(TAG, "new " + cName + " class failed: " + e);
            e.printStackTrace();
        }
        return retObject;
    }

    /**
     * Invoke class member method,disregard private,protected and default
     * @param obj : class object
     * @param mName : method name
     * @param types : type array of parameters
     * @param params: value array of parameters
     * @return : method return value
     */
    public static Object Load(
            Object obj, String mName, Class[] types, Object[] params) {
        Object retObject = null;

        if(obj == null) return null;
        try {
            Class<?> cls = obj.getClass();
            Method meth = null;

            /* Loop to upper class, and get specific member method */
            for(; cls != Object.class; cls = cls.getSuperclass()) {
                try {
                    meth = cls.getDeclaredMethod(mName, types);
                    break;
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
            if (meth != null) {
                /* If it is private authority, modify access limit */
                if (!meth.isAccessible()) meth.setAccessible(true);
                retObject = meth.invoke(obj, params);
            } else {
                Log.e(TAG, "not found " + mName + " method");
            }
        } catch (Exception e) {
            Log.e(TAG, "invoke " + mName  + " method failed: " + e);
        }
        return retObject;
    }

    public static int getSmartVisionType(Context context) {
        int type = SharedPrefsStore.getIntData(context, C.KEY_SMARTVISION_TYPE, C.SMARTVISION_DISABLE);
        try {
            if (type == C.SMARTVISION_HDR
                    && FeatureManager.getInstance().checkHdrFeatureValid(context, BaseApp.APP_ID)) {
                return C.SMARTVISION_HDR;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return C.SMARTVISION_DISABLE;
    }

    public static String stringForTime(int timeMs) {
        int totalSeconds = timeMs / 1000;

        int seconds = totalSeconds % 60;
        int minutes = (totalSeconds / 60) % 60;
        int hours = totalSeconds / 3600;

        if (hours > 0) {
            return String.format(Locale.getDefault(), "%02d:%02d:%02d", hours, minutes, seconds);
        } else {
            return String.format(Locale.getDefault(), "%02d:%02d", minutes, seconds);
        }
    }

    public static int dp2px(Context context, float dpValue) {
        return (int) TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, dpValue,
                context.getResources().getDisplayMetrics());
    }

    public static int sp2px(Context context, float dpValue) {
        return (int) TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_SP, dpValue,
                context.getResources().getDisplayMetrics());
    }

    private static WindowManager getWindowManager(Context context) {
        return (WindowManager) context.getSystemService(Context.WINDOW_SERVICE);
    }

    private static boolean hasNavigationBar(Context context) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN_MR1) {
            Display display = getWindowManager(context).getDefaultDisplay();
            Point size = new Point();
            Point realSize = new Point();
            display.getSize(size);
            display.getRealSize(realSize);
            return realSize.x != size.x || realSize.y != size.y;
        } else {
            boolean menu = ViewConfiguration.get(context).hasPermanentMenuKey();
            boolean back = KeyCharacterMap.deviceHasKey(KeyEvent.KEYCODE_BACK);
            return !(menu || back);
        }
    }

    private static int getNavigationBarHeight(Context context) {
        if (!hasNavigationBar(context)) {
            return 0;
        }
        Resources resources = context.getResources();
        int resourceId = resources.getIdentifier("navigation_bar_height",
                "dimen", "android");
        // 获取NavigationBar的高度
        return resources.getDimensionPixelSize(resourceId);
    }

    public static int getScreenWidth(Context context, boolean isIncludeNav) {
        if (isIncludeNav) {
            return context.getResources().getDisplayMetrics().widthPixels + getNavigationBarHeight(context);
        } else {
            return context.getResources().getDisplayMetrics().widthPixels;
        }
    }

    public static int getScreenHeight(Context context, boolean isIncludeNav) {
        if (isIncludeNav) {
            return context.getResources().getDisplayMetrics().heightPixels + getNavigationBarHeight(context);
        } else {
            return context.getResources().getDisplayMetrics().heightPixels;
        }
    }

    public static boolean isScreenEdge(Context context, MotionEvent e) {
        int edgeSize = CommonUtils.dp2px(context, 40);
        return e.getRawX() < edgeSize
                || e.getRawX() > getScreenWidth(context, true) - edgeSize
                || e.getRawY() < edgeSize
                || e.getRawY() > getScreenHeight(context, true) - edgeSize;
    }

    public static boolean checkNetwork(Context context) {
        ConnectivityManager connectivityManager =
                (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo networkInfo = connectivityManager.getActiveNetworkInfo();
        return networkInfo != null && networkInfo.isConnected();
    }

    public static String getReadableErrorMsg(int what, int extra) {
        if (what == IMediaPlayer.MEDIA_ERROR_BDPLAYER_FW) {
            // 播放内核错误
            switch (extra) {
                case -1:
                    return "无权限操作";
                case -2:
                    return "找不到文件";
                case -5:
                    return "I/O错误";
                case -110:
                    // 1、客户端弱网下产生连接超时；2、服务端返还数据慢导致超时；3、http劫持
                    return "网络连接超时";
                case -111:
                    // 拒绝连接，服务端行为，例如：鉴权
                    return "网络请求被拒绝";
                case -101:
                    // Network is unreachable 如断网或者网络切换后，还使用原IP继续链接
                    return "网络不可用";
                case -541478725:
                    return "End of File";
                case -808465656:
                    return "网络请求失败（http 400）";
                case -858797304:
                    return "网络请求失败（http 403）";
                case -875574520:
                    return "网络请求失败（http 404）";
                case -1482175992:
                    return "网络请求失败（http 50x）";
                case -1094995529:
                    return "解码信息异常";
                case -1414092869:
                    return "数据读取超时";
                case 10002:
                    return "硬件解码器错误";
                default:
                    return "播放内核错误（" + extra + ")";
            }
        } else if (what == IMediaPlayer.MEDIA_ERROR_AUTH_FAILURE) {
            // 鉴权错误
            switch (extra) {
                case 20003:
                    return "鉴权服务器无响应";
                case 20004:
                    return "找不到证书";
                case 20005:
                    return "证书校验错误";
                case 20006:
                    return "证书过期";
                case 20008:
                    return "鉴权参数错误";
                case 20009:
                default:
                    return "鉴权错误（" + extra + ")";
            }
        } else {
            switch (what) {
                case IMediaPlayer.MEDIA_ERROR_IO:
                    return "I/O错误(-1004)";
                case IMediaPlayer.MEDIA_ERROR_MALFORMED:
                    return "媒体格式错误(-1007)";
                case IMediaPlayer.MEDIA_ERROR_UNSUPPORTED:
                    return "不支持播放当前媒体(-1010)";
                case IMediaPlayer.MEDIA_ERROR_TIMED_OUT:
                    return "网络连接超时(-110)";
                default:
                    return "未知错误（" + what + ", " + extra + ")";
            }
        }
    }

    public static Bitmap getBitmapFromAsset(Context context, String filePath) {
        AssetManager assetManager = context.getAssets();
        InputStream istr;
        Bitmap bitmap = null;
        try {
            istr = assetManager.open(filePath);
            bitmap = BitmapFactory.decodeStream(istr);
        } catch (IOException e) {
            e.printStackTrace();
        }

        return bitmap;
    }

}
