package com.baidu.cloud.videoplayer.demo.helper;

import android.content.Context;
import android.view.Surface;

import com.baidu.cloud.media.player.BDCloudMediaMeta;
import com.baidu.cloud.media.player.IMediaPlayer;

public class HdrKitWrapper {
    public static final String HDR_COLOR_PRIMARY = "bt2020";
    public static final String HDR10_COLOR_TRANSFER = "smpte2084";
    public static final String HLG_COLOR_TRANSFER = "arib-std-b67";

    private static final String HDRKIT_CLASS_NAME = "com.baidu.cloud.hdrkit.HDRKit";
    private static final String HDRKIT_CONTENT_TYPE_NAME = "com.baidu.cloud.hdrkit.HDRKit$ContentType";
    private static final String HDRINFO_CLASS_NAME = "com.baidu.cloud.hdrkit.HDRInfo";

    public enum ContentType {
        HDR10(0),
        HLG(1),
        VIVID(2),
        SDR(3);

        private final int value;
        ContentType(int value) {
            this.value = value;
        }

        public int toInt() {
            return this.value;
        }
    }

    public static boolean isHdr10VideoStream(IMediaPlayer mediaPlayer) {
        if (mediaPlayer.getMediaInfo().mMeta != null && mediaPlayer.getMediaInfo().mMeta.mVideoStream != null) {
            return isHdr10VideoStream(mediaPlayer.getMediaInfo().mMeta.mVideoStream);
        }
        return false;
    }

    private static boolean isHdr10VideoStream(BDCloudMediaMeta.BDCloudStreamMeta cloudStreamMeta) {
        return HDR_COLOR_PRIMARY.equalsIgnoreCase(cloudStreamMeta.mColorPrimary)
                && HDR10_COLOR_TRANSFER.equalsIgnoreCase(cloudStreamMeta.mColorTransfer);
    }

    public static boolean isHlgVideoStream(IMediaPlayer mediaPlayer) {
        if (mediaPlayer.getMediaInfo().mMeta != null && mediaPlayer.getMediaInfo().mMeta.mVideoStream != null) {
            return isHlgVideoStream(mediaPlayer.getMediaInfo().mMeta.mVideoStream);
        }
        return false;
    }

    private static boolean isHlgVideoStream(BDCloudMediaMeta.BDCloudStreamMeta cloudStreamMeta) {
        return HDR_COLOR_PRIMARY.equalsIgnoreCase(cloudStreamMeta.mColorPrimary)
                && HLG_COLOR_TRANSFER.equalsIgnoreCase(cloudStreamMeta.mColorTransfer);
    }

    public static boolean isHdr10CodecSupported(Context context) {
        Class[] types = {Context.class};
        Object[] params = {context};
        return (boolean) CommonUtils.invokeStaticMethod(HDRINFO_CLASS_NAME, "isHdr10CodecSupported", types, params);
    }

    public static void initHDRDeviceInfo(Context context) {
        Class[] types = {Context.class};
        Object[] params = {context};
        CommonUtils.invokeStaticMethod(HDRINFO_CLASS_NAME, "initHDRDeviceInfo", types, params);
    }

    public static boolean isHdr10DisplaySupported(Context context) {
        Class[] types = {Context.class};
        Object[] params = {context};
        return (boolean) CommonUtils.invokeStaticMethod(HDRINFO_CLASS_NAME, "isHdr10DisplaySupported", types, params);
    }

    public static boolean isHlgDisplaySupported(Context context) {
        Class[] types = {Context.class};
        Object[] params = {context};
        return (boolean) CommonUtils.invokeStaticMethod(HDRINFO_CLASS_NAME, "isHlgDisplaySupported", types, params);
    }

    public static Object constructHdrKit() {
        Class[] types = {};
        Object[] params = {};

        return CommonUtils.Load(HDRKIT_CLASS_NAME, types, params);
    }

    public static void init(Object hdrkitObject, Context context, String appid) {
        try {
            Class[] types = {Context.class, String.class};
            Object[] params = {context, appid};
            CommonUtils.Load(hdrkitObject, "init", types, params);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void release(Object hdrkitObject) {
        try {
            Class[] types = {};
            Object[] params = {};
            CommonUtils.Load(hdrkitObject, "release", types, params);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void setInputVideoContentType(Object hdrkitObject, ContentType contentType) {
        try {
            Class clazzContentTypeEnum= Class.forName(HDRKIT_CONTENT_TYPE_NAME);
            Enum enumContentType = Enum.valueOf(clazzContentTypeEnum, contentType.name());

            Class[] types = {clazzContentTypeEnum};
            Object[] params = {enumContentType};
            CommonUtils.Load(hdrkitObject, "setInputVideoContentType", types, params);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void prepareWithTargetSurface(Object hdrkitObject, Surface surface) {
        try {
            Class[] types = {Surface.class};
            Object[] params = {surface};
            CommonUtils.Load(hdrkitObject, "prepareWithTargetSurface", types, params);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static Surface getInputSurface(Object hdrkitObject) {
        try {
            Class[] types = {};
            Object[] params = {};
            return (Surface) CommonUtils.Load(hdrkitObject, "getInputSurface", types, params);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public static void setInputSize(Object hdrkitObject, int inputWidth, int inputHeight) {
        try {
            Class[] types = {Integer.TYPE, Integer.TYPE};
            Object[] params = {inputWidth, inputHeight};
            CommonUtils.Load(hdrkitObject, "setInputSize", types, params);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void setOutputSize(Object hdrkitObject, int outputWidth, int outputHeight) {
        try {
            Class[] types = {Integer.TYPE, Integer.TYPE};
            Object[] params = {outputWidth, outputHeight};
            CommonUtils.Load(hdrkitObject, "setOutputSize", types, params);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void startRenderFromSurface(Object hdrkitObject) {
        try {
            Class[] types = {};
            Object[] params = {};
            CommonUtils.Load(hdrkitObject, "startRenderFromSurface", types, params);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void prepareWithInputTexId(Object hdrkitObject, int texid) {
        try {
            Class[] types = {Integer.TYPE};
            Object[] params = {texid};
            CommonUtils.Load(hdrkitObject, "prepareWithInputTexId", types, params);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void setOutputTexId(Object hdrkitObject, int outputTexId) {
        try {
            Class[] types = {Integer.TYPE};
            Object[] params = {outputTexId};
            CommonUtils.Load(hdrkitObject, "setOutputTexId", types, params);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void drawFrameFromTexId(Object hdrkitObject) {
        try {
            Class[] types = {};
            Object[] params = {};
            CommonUtils.Load(hdrkitObject, "drawFrameFromTexId", types, params);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void setHdrStaticMetadata(Object hdrkitObject, int metadataType, byte[] metadata) {
        try {
            Class[] types = {Integer.TYPE, byte[].class};
            Object[] params = {metadataType, metadata};
            CommonUtils.Load(hdrkitObject, "setHdrStaticMetadata", types, params);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void setVividMetadata(Object hdrkitObject, byte[] metadata) {
        try {
            Class[] types = {byte[].class};
            Object[] params = {metadata};
            CommonUtils.Load(hdrkitObject, "setVividMetadata", types, params);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
