package com.baidu.cloud.videoplayer.demo.info;

import org.json.JSONArray;
import org.json.JSONObject;

import android.os.Parcel;
import android.os.Parcelable;

/**
 * 视频信息类
 * @author baidu
 *
 */
public class VideoInfo implements Parcelable {
    
    private String title = "";
    private String url = "";
    private String imageUrl = "";
    private String[] preImages = new String[16];
    private boolean canDelete = true;
    
    private int demoEnable = 1;
    private int switchMode;
    private String[] inputList = new String[3];
    private String[] mediasDes = new String[3];
    private boolean vrEnable;
    private boolean hdrEnable;

    public VideoInfo(String title, String url) {
        super();
        this.title = title;
        this.url = url;
    }
    
    public String getTitle() {
        return title;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public String getUrl() {
        return url;
    }

    public void setUrl(String url) {
        this.url = url;
    }

    public String getImageUrl() {
        return imageUrl;
    }

    public void setImageUrl(String imageUrl) {
        this.imageUrl = imageUrl;
    }

    public String[] getPreImages() {
        return preImages;
    }
    public void setPreImages(String[] preImages) {
        this.preImages = preImages;
    }

    public boolean isCanDelete() {
        return canDelete;
    }

    public void setCanDelete(boolean canDelete) {
        this.canDelete = canDelete;
    }

    public void setDemoEnable(int demoEnable) {
        this.demoEnable = demoEnable;
    }

    public int getDemoEnable() {
        return demoEnable;
    }

    public void setSwitchMode(int switchMode) {
        this.switchMode = switchMode;
    }

    public int getSwitchMode() {
        return switchMode;
    }

    public void setInputList(String[] inputList) {
        this.inputList = inputList;
    }

    public String[] getInputList() {
        return inputList;
    }

    public String[] getMediasDes() {
        return mediasDes;
    }

    public void setMediasDes(String[] mediasDes) {
        this.mediasDes = mediasDes;
    }

    public boolean isVrEnable() {
        return vrEnable;
    }

    public void setVrEnable(boolean vrEnable) {
        this.vrEnable = vrEnable;
    }

    public boolean isHdrEnable() {
        return hdrEnable;
    }

    public void setHdrEnable(boolean hdrEnable) {
        this.hdrEnable = hdrEnable;
    }

    /**
     * 方便持久化
     * @return
     */
    public JSONObject toJson() {
        JSONObject json = new JSONObject();
        try {
            json.put("url", url);
            json.put("title", title);
            json.put("imageUrl", imageUrl);
            json.put("preImages", new JSONArray(preImages));
            json.put("canDelete", canDelete);
            json.put("demoEnable", demoEnable);
            json.put("switchMode", switchMode);
            json.put("inputList", new JSONArray(inputList));
            json.put("mediasDes", new JSONArray(mediasDes));
            json.put("vrEnable", vrEnable);
            json.put("hdrEnable", hdrEnable);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return json;
    }
    
    /**
     * 从持久化恢复
     * @param json
     * @return
     */
    public static VideoInfo fromJson(JSONObject json) {
        VideoInfo info = null;
        try {
            info = new VideoInfo(json.getString("title"), json.getString("url"));
            info.setImageUrl(json.optString("imageUrl", ""));
            JSONArray preImagesArray = json.optJSONArray("preImages");
            info.setPreImages(toStringArray(preImagesArray));
            info.setCanDelete(json.optBoolean("canDelete", true));
            info.setDemoEnable(json.optInt("demoEnable"));
            info.setSwitchMode(json.optInt("switchMode", 0));
            JSONArray inputListArray = json.optJSONArray("inputList");
            info.setInputList(toStringArray(inputListArray));
            JSONArray mediaDesArray = json.optJSONArray("mediasDes");
            info.setMediasDes(toStringArray(mediaDesArray));
            info.setVrEnable(json.optBoolean("vrEnable", false));
            info.setHdrEnable(json.optBoolean("hdrEnable", false));
        } catch (Exception e) {
            e.printStackTrace();
        }
        return info;
    }

    
    /**
     * 以下三个函数或成员，都属于Parcelable接口。是为了方便通过Intent传输准备的。
     */
    @Override
    public int describeContents() {
        return 0;
    }

    /**
     * Intent putExtra时，会调用的函数
     */
    @Override
    public void writeToParcel(Parcel dest, int flags) {
        dest.writeString(title);
        dest.writeString(url);
        dest.writeString(imageUrl);
        dest.writeStringArray(preImages);
        boolean[] boolArray = new boolean[1];
        boolArray[0] = this.canDelete;
        dest.writeBooleanArray(boolArray);
        dest.writeInt(demoEnable);
        dest.writeInt(switchMode);
        dest.writeStringArray(inputList);
        dest.writeStringArray(mediasDes);
        boolean[] vrBoolArray = new boolean[1];
        vrBoolArray[0] = this.vrEnable;
        dest.writeBooleanArray(vrBoolArray);
        boolean[] hdrBoolArray = new boolean[1];
        hdrBoolArray[0] = this.hdrEnable;
        dest.writeBooleanArray(hdrBoolArray);
    }
    
    /**
     * Intent getExtra时，
     */
    public static final Parcelable.Creator<VideoInfo> CREATOR = new Creator<VideoInfo>() {

        @Override
        public VideoInfo createFromParcel(Parcel source) {
            String title = source.readString();
            String url = source.readString();
            String imageUrl = source.readString();
            String[] preImages = new String[16];
            source.readStringArray(preImages);
            boolean[] boolArray = new boolean[1];
            source.readBooleanArray(boolArray);
            int demoEnable = source.readInt();
            int switchMode = source.readInt();
            String[] inputList = new String[3];
            source.readStringArray(inputList);
            String[] mediasDes = new String[3];
            source.readStringArray(mediasDes);
            boolean[] vrBoolArray = new boolean[1];
            source.readBooleanArray(vrBoolArray);
            boolean[] hdrBoolArray = new boolean[1];
            source.readBooleanArray(hdrBoolArray);

            VideoInfo p = new VideoInfo(title, url);
            p.setImageUrl(imageUrl);
            p.setPreImages(preImages);
            p.setCanDelete(boolArray[0]);
            p.setDemoEnable(demoEnable);
            p.setSwitchMode(switchMode);
            p.setInputList(inputList);
            p.setMediasDes(mediasDes);
            p.setVrEnable(vrBoolArray[0]);
            p.setHdrEnable(hdrBoolArray[0]);
            return p;
        }

        @Override
        public VideoInfo[] newArray(int size) {
            return new VideoInfo[size];
        }
    };

    private static String[] toStringArray(JSONArray array) {
        if(array == null)
            return new String[0];

        String[] arr = new String[array.length()];
        for(int i = 0; i < arr.length; i++) {
            arr[i] = array.optString(i);
        }
        return arr;
    }
}
