package com.baidu.cloud.videoplayer.demo.popview;

import com.baidu.cloud.videoplayer.demo.R;

import android.app.Activity;
import android.content.Context;
import android.graphics.drawable.ColorDrawable;
import android.util.DisplayMetrics;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.LinearLayout;
import android.widget.LinearLayout.LayoutParams;
import android.widget.ListView;
import android.widget.PopupWindow;

public class DefaultPopWindow extends PopupWindow {
    private static final int DEFAULT_SELECTOR_WIDTH_DPS = 55;
    private View contentView;
    private DisplayMetrics dm = new DisplayMetrics();

    private DefaultPopWindow(final Activity context, View ctView,
                             String[] optionName, final OnItemClickListener listener) {
        this(context, ctView, optionName, DEFAULT_SELECTOR_WIDTH_DPS, listener);
    }

    private DefaultPopWindow(final Activity context, View ctView,
                             String[] optionName, int selectorWidthDps, final OnItemClickListener listener) {
        super(ctView, LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT, true);
        contentView = ctView;
        context.getWindowManager().getDefaultDisplay().getMetrics(dm);
        // 设置SelectPicPopupWindow的View
//        this.setContentView(contentView);
        // // 设置SelectPicPopupWindow弹出窗体的宽
//        this.setWidth(LayoutParams.WRAP_CONTENT);
//        // // 设置SelectPicPopupWindow弹出窗体的高
//        this.setHeight(LayoutParams.WRAP_CONTENT);
//        // 设置SelectPicPopupWindow弹出窗体可点击
//        this.setFocusable(true);
        this.setOutsideTouchable(true);
        // 刷新状态
        this.update();
        // 实例化一个ColorDrawable颜色为半透明
        ColorDrawable dw = new ColorDrawable(0x000000);
        // 点back键和其他地方使其消失,设置了这个才能触发OnDismisslistener ，设置其他控件变化等操作
        this.setBackgroundDrawable(dw);
        this.setAnimationStyle(android.R.style.Animation_Dialog);

        ListView mSelector = contentView.findViewById(R.id.selector);
        mSelector.setLayoutParams(new LinearLayout.LayoutParams(dpToPx(dm.density, selectorWidthDps),
                ViewGroup.LayoutParams.WRAP_CONTENT));

        mSelector.setAdapter(new ArrayAdapter<>(context, R.layout.item_of_list_default_item, optionName));
        mSelector.setOnItemClickListener(new OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                listener.onItemClick(parent, view, position, id);
                dismiss();
            }
        });
    }

    public static DefaultPopWindow createOptionListPopWindow(final Activity context,
                                                             String[] options,
                                                             final OnItemClickListener listener) {
        return createOptionListPopWindow(context, options, DEFAULT_SELECTOR_WIDTH_DPS, listener);
    }

    public static DefaultPopWindow createOptionListPopWindow(final Activity context,
                                                             String[] options,
                                                             int widthDps,
                                                             final OnItemClickListener listener) {
        LayoutInflater inflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        View ctView = inflater.inflate(R.layout.pop_window_defaultlist, null);
        return new DefaultPopWindow(context, ctView, options, widthDps, listener);
    }

    @Override
    public void dismiss() {
        super.dismiss();
    }

    /**
     * 显示popupWindow
     */
    public void showPopupWindow(View view) {
        showPopupWindowAt(view, Gravity.BOTTOM, (int) (dm.widthPixels * 2 / 7),
                (int) (dm.density * 54));
    }

    public void showPopupWindowAt(View view, int gravity, int x, int y) {
        if (!this.isShowing()) {
            // dm.widthPixels - (int)(dm.density * 165)
            // 以下拉方式显示popupwindow
            this.showAtLocation(view, gravity, x, y);
        } else {
            this.dismiss();
        }
    }

    private static int dpToPx(float density, int dps) {
        return (int) (dps * density + 0.5f);
    }
}
